SystemOrganization addCategory: #'FS-Git-Plumbing'!
SystemOrganization addCategory: #'FS-Git-Plumbing-Tests'!
SystemOrganization addCategory: #'FS-Git'!
SystemOrganization addCategory: #'FS-Git-Tests'!
SystemOrganization addCategory: #'FS-Git-Notifications'!

Notification subclass: #FSGitCheckOutRequest
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Notifications'!

!FSGitCheckOutRequest classSide methodsFor: 'exceptioninstantiator' stamp: 'MaxLeske 7/25/2010 17:17'!
signalWith: aReference
	^ self new 
		reference: aReference;
		signal! !

!FSGitCheckOutRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/28/2010 11:41'!
defaultAction
	^ StandardWindow new questionWithoutCancel: 
		'All files currently in ', self reference basename, ' will be overridden. Continue?'! !

!FSGitCheckOutRequest methodsFor: 'accessing' stamp: 'MaxLeske 7/25/2010 17:07'!
reference
	^ reference! !

!FSGitCheckOutRequest methodsFor: 'accessing' stamp: 'MaxLeske 7/25/2010 17:07'!
reference: aReference
	reference := aReference
	! !

Notification subclass: #FSGitResetRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Notifications'!

!FSGitResetRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 16:15'!
defaultAction
	^ StandardWindow 	new questionWithoutCancel: 
		'You will loose all changes that you didn''t commit. Do you really want the modifications to be reset?'! !

Notification subclass: #FSGitSwitchBranchRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Notifications'!

!FSGitSwitchBranchRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 16:18'!
defaultAction
	^ StandardWindow new questionWithoutCancel: 
		'You will loose all changes you didn''t commit. Do you really want to switch branches?'! !

Notification subclass: #GitRepositoryDeleteRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitRepositoryDeleteRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 15:12'!
defaultAction
	^ StandardWindow new questionWithoutCancel: 
		'Do you really want to remove the repository? This cannot be undone!!'! !

Notification subclass: #GitStampEmailRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStampEmailRequest commentStamp: '<historical>' prior: 0!
I provide a mechanism to handle a request for the e-mail address used in a GitStamp object before it triggers an input form. This is especially useful for testing purposes and scripts.!

!GitStampEmailRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/13/2010 14:50'!
defaultAction
	| mail |
	mail := GitStamp requestEmail.
	GitStamp defaultEmail: mail.
	^ mail! !

Notification subclass: #GitStampNameRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStampNameRequest commentStamp: '<historical>' prior: 0!
I provide a mechanism to handle a request for the name used in a GitStamp object before it triggers an input form. This is especially useful for testing purposes and scripts.!

!GitStampNameRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/13/2010 14:50'!
defaultAction
	| name |
	name := GitStamp requestName.
	GitStamp defaultName: name.
	^ name! !

FSMemoryFilesystem subclass: #FSGitFilesystem
	instanceVariableNames: 'repo revision cache author committer modManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitFilesystem commentStamp: '<historical>' prior: 0!
I work on a certain commit (revision) of a repository (in git speak: I represent a checked out version of a repository). I provide access to a filesystem containing the working copy which means that files and directories of the working copy can be manipulated with the same tools used by Colin Putney's Filesystem.!

!FSGitFilesystem class methodsFor: 'initialize-release' stamp: 'MaxLeske 10/11/2010 19:42'!
for: aRepository
	^ self basicNew initializeOn: aRepository! !

!FSGitFilesystem class methodsFor: 'initialize-release' stamp: 'MaxLeske 10/11/2010 19:50'!
new
	self error: 'Please use the constructor methods to create instances.'! !

!FSGitFilesystem methodsFor: 'git' stamp: 'MaxLeske 10/12/2010 21:54'!
activeBranch
"Answers the name of the currently active branch which is the branch referenced in HEAD. This method can answer nil."

	| headContents |
	headContents := repo headFile readStreamDo: [ :stream | stream contents asString withBlanksTrimmed ].
	(headContents beginsWith: 'ref:')
		ifTrue: [ ^ (headContents subStrings: '/') last ]
		ifFalse: [ ^ nil ]! !

!FSGitFilesystem methodsFor: 'internal' stamp: 'MaxLeske 10/17/2010 22:06'!
basicCommit: aMessage to: branchName
	| commit stamp parents | 
	"Disallow empty commits"
	self assert: modManager hasModifications.

	modManager processBlobs.
	modManager processDeletions.
	modManager storeTrees.
	
	commit := GitCommit
		withTree: modManager root
		message: aMessage
		author: (author ifNil: [ author := GitStamp now ])
		committer: (committer ifNil: [ committer := GitStamp now ]).
	parents := OrderedCollection new.
	revision ifNotNil: [ parents add: revision ].
	commit parents: parents.
	repo storeObject: commit.
	repo updateRef: repo headsDir / branchName to: commit signature.
	"revision := commit."
	
	self initializeOn: repo! !

!FSGitFilesystem methodsFor: 'internal' stamp: 'MaxLeske 10/17/2010 22:25'!
basicLookup: aPath
	| current entry |
	current := modManager root
		ifNil: [ repo head 
			ifNil: [ modManager newTreeAt: aPath ]
			ifNotNil: [ repo head tree ] ].
	aPath do: [ :segment |
		current species = GitTree ifTrue: [
			entry := current
				entryAt: segment
				ifAbsent: [ ^ self ]. 
				current := repo objectWithSignature: entry objectSignature ] ].
		
	current species = GitTree
		ifTrue: [ cache ensureDirectory: aPath ]
		ifFalse: [ (cache referenceTo: aPath) 
			writeStreamDo: [ :stream | stream nextPutAll: current bytes ] ]! !

!FSGitFilesystem methodsFor: 'git' stamp: 'MaxLeske 10/12/2010 17:29'!
commit: aMessage
"Creates a commit with message ==aMessage== in the currently active branch."

	self basicCommit: aMessage to: self activeBranch! !

!FSGitFilesystem methodsFor: 'private' stamp: 'MaxLeske 10/17/2010 20:00'!
createFile: path
	| entry blob |
	^ self 
		nodeAt: path parent
		ifPresent: 
			[ :dict | 
			(self basicIsDirectory: dict) ifTrue: 
				[ dict 
					at: path basename
					put: ByteArray new.
				self basicLookup: path parent.
				^ dict ] ]
		ifAbsent: [ self directoryDoesNotExist: path parent ]! !

!FSGitFilesystem methodsFor: 'private' stamp: 'MaxLeske 10/12/2010 08:01'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock 
	self 
		nodeAt: aPath
		ifPresent: [ :directory | 
			directory isDictionary
				ifFalse: [ ^ absentBlock value ].
			self lookupDirectory: aPath.
			directory associations sorted
				do: aBlock ]
		ifAbsent: absentBlock! !

!FSGitFilesystem methodsFor: 'initialize-release' stamp: 'MaxLeske 10/17/2010 17:15'!
initializeOn: aRepository
	super initialize.
	repo := aRepository.
	revision ifNil: [ revision := repo head ].
	cache := FSMemoryFilesystem new.
	modManager := FSGitModificationManager new.
	modManager repository: repo.
	modManager cache: cache! !

!FSGitFilesystem methodsFor: 'internal' stamp: 'MaxLeske 10/12/2010 17:42'!
lookup: aPath
	| current |
	(cache exists: aPath) ifTrue: [ ^ aPath ].
	
	self basicLookup: aPath.	
	^ aPath! !

!FSGitFilesystem methodsFor: 'internal' stamp: 'MaxLeske 10/17/2010 19:55'!
lookupDirectory: aPath
	| current ref |
	revision
		ifNil: [ current := GitTree new ]
		ifNotNil: [ current := revision tree ].
	aPath do: [ :segment |
		current species = GitTree ifTrue: [
			current := repo objectWithSignature: 
				(current
					entryAt: segment
					ifAbsent: [ ^ self fileDoesNotExist: aPath ]) objectSignature ] ].
		cache ensureDirectory: aPath.
		ref := cache referenceTo: aPath.
		current entries do: [ :entry |
			(ref / entry entryName) exists
				ifFalse: [
					entry mode = GitTreeEntry dirMode
					ifTrue: [ (ref / entry entryName) ensureDirectory ]
					ifFalse: [ 
						(ref / entry entryName) writeStreamDo: [ :stream | 
							stream nextPutAll: (repo objectWithSignature: entry objectSignature) bytes  ] ] ] ]! !

!FSGitFilesystem methodsFor: 'private' stamp: 'MaxLeske 10/11/2010 23:24'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	^ cache
		nodeAt: (self lookup: aPath)
		ifPresent: presentBlock
		ifAbsent: absentBlock! !

!FSGitFilesystem methodsFor: 'private' stamp: 'MaxLeske 10/17/2010 17:29'!
open: aPath writable: boolean
	| handle path tree |
	handle := super
		open: aPath
		writable: boolean.
	boolean ifTrue: [ 
		path := aPath.
		[ path = FSPath root ] whileFalse: [
			modManager
				markCreated: path
				inTree: (modManager treeAt: path parent).
			path := path parent ] ].
	
	^ handle! !

FSFilesystemTest subclass: #FSGitFilesystemTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitFilesystemTests class methodsFor: 'accessing' stamp: 'MaxLeske 7/24/2010 17:26'!
packageNamesUnderTest
	^ #('FS-Git')! !

!FSGitFilesystemTests methodsFor: 'initialize-release' stamp: 'MaxLeske 7/21/2010 20:58'!
createFilesystem
	^ (FSGitRepository on: (FSMemoryFilesystem new root resolve: 'testRepo.git')) head.! !

TestCase subclass: #FSGitFilesystemUsageTests
	instanceVariableNames: 'basicRepository fs modified workingCopy fsRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 7/18/2010 13:43'!
branch: branchName message: aMessage
	[ [ fs branch: branchName message: aMessage ] 
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 7/16/2010 10:09'!
commit: message
	[ [ fs commit: message ]
		on: GitStampNameRequest 
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 7/18/2010 13:53'!
commit: aMessage andTag: tagName
	[ [ fs commit: aMessage andTag: tagName ] 
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 7/21/2010 18:33'!
createSomeCommits
	(fs root resolve: 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'content' ].
	self commit: 'first commit'.
	(fs root resolve: 'dir1') ensureDirectory.
	(fs root resolve: 'dir1/file2') writeStreamDo: [ :stream | stream nextPutAll: 'more content' ].
	self commit: 'second commit'! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 7/24/2010 16:47'!
reset
	[ fs reset ]
		on: FSGitResetRequest 
		do: [ :req | req resume: true ]! !

!FSGitFilesystemUsageTests methodsFor: 'running' stamp: 'MaxLeske 7/21/2010 20:54'!
setUp
	| ref |
	ref := FSMemoryFilesystem new root resolve: 'testRepo'.
	basicRepository := GitRepository on: ref.
	fsRepository := FSGitRepository  on: ref.
	fs := fsRepository head.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/5/2010 14:08'!
testActiveBranch
	self assert: fs activeBranch = 'master'.
	(fs root / 'aFile') writeStreamDo: [ :stream | ].
	self branch: 'branch 1' message: 'branch 1 commit'.
	self assert: fs activeBranch = 'branch 1'.
	basicRepository updateHeadToBranch: 'master'.
	self assert: fs activeBranch = 'master'.
	
	(fs root / 'anotherFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	basicRepository 
		basicUpdateRef: basicRepository headFile 
		to: basicRepository head signature hexHash.
	self assert: fs activeBranch isNil! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/21/2010 18:16'!
testAdding
	| ref filename |
	self assert: fs root children isEmpty.
	self assert: workingCopy root children isEmpty.
	self assert: modified root children isEmpty.
	
	filename := 'aFile'.
	ref := fs root resolve: filename.
	self deny: ref exists.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	self assert: (modified root resolve: filename) exists.
	self deny: (workingCopy root resolve: filename) exists.
	
	self commit: 'a commit'.
	self assert: ref exists.
	self deny: (modified root resolve: filename) exists.
	self assert: (workingCopy root resolve: filename) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:42'!
testAddingMultipleDirectories
	| dir1 subDir1 dir2 |
	dir1 := 'aDir'.
	subDir1 := dir1 , '/sub'.
	dir2 := 'dir 2'.
	(fs root resolve: dir1) createDirectory.
	self assert: (fs root resolve: dir1) exists.
	self assert: (modified root resolve: dir1) exists.
	self deny: (workingCopy root resolve: dir1) exists.
	
	(fs root resolve: subDir1) createDirectory.
	self assert: (fs root resolve: subDir1) exists.
	self assert: (modified root resolve: subDir1) exists.
	self deny: (workingCopy root resolve: subDir1) exists.
	
	(fs root resolve: dir2) createDirectory.
	self assert: (fs root resolve: dir2) exists.
	self assert: (modified root resolve: dir2) exists.
	self deny: (workingCopy root resolve: dir2) exists.
	
	self commit: 'aCommit'.
	self assert: (fs root resolve: dir1) exists.
	self deny: (modified root resolve: dir1) exists.
	self assert: (workingCopy root resolve: dir1) exists.
	
	self assert: (fs root resolve: subDir1) exists.
	self deny: (modified root resolve: subDir1) exists.
	self assert: (workingCopy root resolve: subDir1) exists.
	
	self assert: (fs root resolve: dir2) exists.
	self deny: (modified root resolve: dir2) exists.
	self assert: (workingCopy root resolve: dir2) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:44'!
testAddingMultipleDirectoriesWithReload
	| dir1 subDir1 dir2 |
	dir1 := 'aDir'.
	subDir1 := dir1 , '/sub'.
	dir2 := 'dir 2'.
	(fs root resolve: dir1) createDirectory.
	self assert: (fs root resolve: dir1) exists.
	self assert: (modified root resolve: dir1) exists.
	self deny: (workingCopy root resolve: dir1) exists.
	
	(fs root resolve: subDir1) createDirectory.
	self assert: (fs root resolve: subDir1) exists.
	self assert: (modified root resolve: subDir1) exists.
	self deny: (workingCopy root resolve: subDir1) exists.
	
	(fs root resolve: dir2) createDirectory.
	self assert: (fs root resolve: dir2) exists.
	self assert: (modified root resolve: dir2) exists.
	self deny: (workingCopy root resolve: dir2) exists.
	
	self commit: 'aCommit'.
	fs := fsRepository head.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	self assert: (fs root resolve: dir1) exists.
	self deny: (modified root resolve: dir1) exists.
	self assert: (workingCopy root resolve: dir1) exists.
	
	self assert: (fs root resolve: subDir1) exists.
	self deny: (modified root resolve: subDir1) exists.
	self assert: (workingCopy root resolve: subDir1) exists.
	
	self assert: (fs root resolve: dir2) exists.
	self deny: (modified root resolve: dir2) exists.
	self assert: (workingCopy root resolve: dir2) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:37'!
testAddingSingleDirectory
	| ref dirname |
	dirname := 'aDir'.
	ref := fs root resolve: dirname.
	ref createDirectory.
	self assert: ref exists.
	self assert: (modified root resolve: dirname) exists.
	self deny: (workingCopy root resolve: dirname) exists.
	
	self commit: 'aCommit'.
	self assert: ref exists.
	self deny: (modified root resolve: dirname) exists.
	self assert: (workingCopy root resolve: dirname) exists.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:43'!
testAddingSingleDirectoryWithReload
	| ref dirname |
	dirname := 'aDir'.
	ref := fs root resolve: dirname.
	ref createDirectory.
	self assert: ref exists.
	self assert: (modified root resolve: dirname) exists.
	self deny: (workingCopy root resolve: dirname) exists.
	
	self commit: 'aCommit'.
	fs := fsRepository head.
	ref := fs root resolve: dirname.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	self assert: ref exists.
	self deny: (modified root resolve: dirname) exists.
	self assert: (workingCopy root resolve: dirname) exists.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:08'!
testAddingWithReload
	| ref filename |
	self assert: fs root children isEmpty.
	self assert: workingCopy root children isEmpty.
	self assert: modified root children isEmpty.
	
	filename := 'aFile'.
	ref := fs root resolve: filename.
	self deny: ref exists.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	self assert: (modified root resolve: filename) exists.
	self deny: (workingCopy root resolve: filename) exists.
	
	self commit: 'a commit'.
	fs := fsRepository head.
	ref := fs root resolve: filename.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	self assert: ref exists.
	self deny: (modified root resolve: filename) exists.
	self assert: (workingCopy root resolve: filename) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:15'!
testBranching
	| master branch |
	(fs root / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	master := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/master').
	self assert: master message = 'a commit'.
	self assert: fs activeBranch = 'master'.
	
	(fs root / 'anotherFile') writeStreamDo: [ :stream | ].
	self branch: 'my fork' message: 'my first branch'.
	branch := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/my fork').
	self assert: branch message = 'my first branch'.
	self assert: fs activeBranch = 'my fork'.
	
	self assert: branch parents size = 1.
	self assert: branch parents first = master
	! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/25/2010 17:37'!
testCheckOutTo
	| node1 node2 node3 |
	self createSomeCommits.
	node1 := basicRepository workingDir / 'file1'.
	node2 := basicRepository workingDir / 'dir1'.
	node3 := basicRepository workingDir resolve: 'dir1/file2'.
	self deny: node1 exists.
	self deny: node2 exists.
	self deny: node3 exists.
	
	[ fs checkOutTo: basicRepository workingDir ]
		on: FSGitCheckOutRequest 
		do: [ :req | req resume: true ].
	self assert: node1 exists.
	self assert: node2 exists.
	self assert: node3 exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/25/2010 17:39'!
testCheckOutToWithOverride
	| node1 node2 node3 |
	self createSomeCommits.
	node1 := basicRepository workingDir / 'file1'.
	node2 := basicRepository workingDir / 'dir1'.
	node3 := basicRepository workingDir resolve: 'dir1/file2'.
	self deny: node1 exists.
	self deny: node2 exists.
	self deny: node3 exists.
	node1 writeStreamDo: [ :stream | stream nextPutAll: 'foo' ].
	self assert: node1 exists.
	
	[ fs checkOutTo: basicRepository workingDir ]
		on: FSGitCheckOutRequest 
		do: [ :req | req resume: true ].
	self assert: node1 exists.
	self assert: node1 readStream contents asString = 'content'.
	self assert: node2 exists.
	self assert: node3 exists.
	self assert: (basicRepository workingDir children includes: basicRepository baseDir)! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 17:08'!
testCommitting
	| objectMap revision |
	objectMap := fs instVarNamed: 'objectMap'.
	revision := fs instVarNamed: 'revision'.
	self deny: (workingCopy root / 'aFile') exists.
	self deny: (modified root / 'aFile') exists.
	self deny: (objectMap includesKey: FSPath root / 'aFile').
	self assert: revision isNil.
	
	(fs root / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1'.
	revision := fs instVarNamed: 'revision'.
	self assert: (workingCopy root / 'aFile') exists.
	self deny: (modified root / 'aFile') exists.
	self assert: (objectMap includesKey: FSPath root / 'aFile').
	self assert: revision isNil not.
	self assert: revision tree entries size = 1.
	self assert: (revision tree entries includesKey: 'aFile').
	
	(fs root / 'file2') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	revision := fs instVarNamed: 'revision'.
	self assert: (workingCopy root / 'file2') exists.
	self assert: (workingCopy root / 'aFile') exists.
	self deny: (modified root / 'file2') exists.
	self deny: (modified root / 'aFile') exists.
	self assert: (objectMap includesKey: FSPath root / 'file2').
	self assert: (objectMap includesKey: FSPath root / 'aFile').
	self assert: revision isNil not.
	self assert: revision tree entries size = 2.
	self assert: (revision tree entries includesKey: 'aFile').
	self assert: (revision tree entries includesKey: 'file2')! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 17:00'!
testCommittingWithReload
	| objectMap |
	objectMap := fs instVarNamed: 'objectMap'.	
	(fs root / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1'.
	self assert: (workingCopy root / 'aFile') exists.
	self deny: (modified root / 'aFile') exists.
	self assert: (objectMap includesKey: FSPath root / 'aFile').
	
	fs := fsRepository head.
	workingCopy := fs instVarNamed: 'workingCopy'.
	modified := fs instVarNamed: 'modified'.
	objectMap := fs instVarNamed: 'objectMap'.
	(fs root / 'file2') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	self assert: (workingCopy root / 'file2') exists.
	self assert: (workingCopy root / 'aFile') exists.
	self deny: (modified root / 'file2') exists.
	self deny: (modified root / 'aFile') exists.
	self assert: (objectMap includesKey: FSPath root / 'file2').
	self assert: (objectMap includesKey: FSPath root / 'aFile')! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/17/2010 22:10'!
testCopying
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := fs root resolve: 'copy'.
	self deny: ref exists.
	
	(fs root resolve: 'file1') copyTo: ref.
	self commit: 'a commit'.
	self assert: ref exists.
	self deny: (modified root resolve: 'copy') exists.
	self deny: (modified root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'copy') exists.
	originalContent := (fs root resolve: 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'.
	! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/17/2010 22:11'!
testCopyingExistingFile
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := fs root resolve: 'copy'.
	self deny: ref exists.
	
	(fs root resolve: 'file1') copyTo: ref.
	self assert: ref exists.
	self assert: (modified root resolve: 'copy') exists.
	self deny: (modified root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'file1') exists.
	self deny: (workingCopy root resolve: 'copy') exists.
	originalContent := (fs root resolve: 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/22/2010 18:11'!
testCopyingWithReload
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := fs root resolve: 'copy'.
	self deny: ref exists.
	
	(fs root resolve: 'file1') copyTo: ref.
	self commit: 'a commit'.
	fs := fsRepository head.
	ref := fs root resolve: 'copy'.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	self assert: ref exists.
	self deny: (modified root resolve: 'copy') exists.
	self deny: (modified root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'copy') exists.
	originalContent := (fs root resolve: 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:12'!
testCreateAndDeleteFile
	| stream |
	self assert: modified root children isEmpty.
	stream := fs writeStreamOn: 'newFile'. 
	[ stream nextPutAll: 'some content'. ]
		ensure: [ stream close ].
	self deny: modified root children isEmpty.
	
	fs delete: (fs resolve: 'newFile').
	self assert: modified root children isEmpty.
	self deny: (fs referenceTo: 'newFile') exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:12'!
testCreateAndReadFile
	| stream filename |
	filename := 'missing File'.
	self deny: (modified referenceTo: filename) exists.
	self deny: (workingCopy referenceTo: filename) exists.
	self
		shouldnt: [ 
			stream := fs writeStreamOn: filename. 
			[ stream nextPutAll: 'initial content'. ]
				ensure: [ stream close ] ]
		raise: FSFileDoesNotExist.
	self assert: (fs readStreamOn: filename) contents asString = 'initial content'.
	self deny: (workingCopy referenceTo: filename) exists.
	self assert: (modified referenceTo: filename) exists.
	self assert: (fs referenceTo: filename) exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/16/2010 10:22'!
testDeleteExistingFile
	self createSomeCommits.
	self assert: fs root children size = 2.
	self assert: fs root allChildren size = 4.
	
	fs delete: (fs resolve: 'file1').
	self assert: fs root children size = 1.
	self assert: fs root allChildren size = 3.
	self deny: (fs root resolve: 'file1') exists.
	self assert: (fs root resolve: 'dir1/file2') exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 19:06'!
testDeleteModifiedFile
	| ref path map deletions |
	path := 'dir1/file2'.
	map := fs instVarNamed: 'objectMap'.
	deletions := fs instVarNamed: 'deletions'.
	self createSomeCommits.
	"self assert: (map includesKey: FSPath root / path)."
	self assert: deletions isEmpty.
	(fs root resolve: path) writeStreamDo: [ :stream | 
		stream nextPutAll: 'changed it' ].
	
	ref := fs root resolve: path.
	self assert: ref readStream contents asString = 'changed it'.
	self assert: (modified root resolve: path) readStream contents asString = 'changed it'.
	self assert: (workingCopy root resolve: path) readStream contents asString = 'more content'.
	
	ref delete.
	self deny: ref exists.
	self deny: (modified root resolve: path) exists.
	self assert: (workingCopy root resolve: path) exists.
	self deny: (fs root resolve: path) exists.
	self deny: (map includesKey: FSPath root / path).
	self deny: deletions isEmpty.
	self assert: deletions size = 1.
	self assert: (deletions includes: (fs resolve: path))! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 18:15'!
testDeleting
	| ref deletions objectMap |
	self createSomeCommits.
	deletions := fs instVarNamed: 'deletions'.
	objectMap := fs instVarNamed: 'objectMap'.
	ref := fs root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: deletions isEmpty.
	
	ref delete.
	self assert: deletions size = 1.
	self deny: ref exists.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self assert: (workingCopy root resolve: 'dir1/file2') exists.
	
	self commit: 'a commit'.
	self deny: ref exists.
	self assert: deletions isEmpty.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self deny: (workingCopy root resolve: 'dir1/file2') exists.
	self assert: (#('/' '/file1' '/dir1' ) 
			includesAllOf: (fs root allChildren collect: [ :each | each path asString ]))! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/3/2010 16:46'!
testDeletingDirectory
	| ref deletions |
	self createSomeCommits.
	deletions := fs instVarNamed: 'deletions'.
	ref := fs root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: deletions isEmpty.
	
	ref delete.
	self assert: deletions size = 1.
	self deny: ref exists.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self assert: (workingCopy root resolve: 'dir1/file2') exists.
	
	self commit: 'a commit'.
	self deny: ref exists.
	self assert: deletions isEmpty.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self deny: (workingCopy root resolve: 'dir1/file2') exists.
	self assert: (#('/' '/file1' '/dir1') 
			includesAllOf: (fs root allChildren collect: [ :each | each path asString ]))! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 17:23'!
testDeletingInDirectoryWithReload
	| ref deletions objectMap |
	self createSomeCommits.
	deletions := fs instVarNamed: 'deletions'.
	objectMap := fs instVarNamed: 'objectMap'.
	ref := fs root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: deletions isEmpty.
	self assert: (objectMap includesKey: FSPath root / 'dir1' / 'file2').
	
	ref delete.
	self assert: deletions size = 1.
	self assert: (deletions includes: FSPath root / 'dir1' / 'file2').
	self deny: ref exists.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self assert: (workingCopy root resolve: 'dir1/file2') exists.
	self assert: (objectMap includesKey: FSPath root / 'dir1' / 'file2'). 
	
	self commit: 'a commit'.
	fs := fsRepository head.
	ref := fs root resolve: 'copy'.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	deletions := fs instVarNamed: 'deletions'.
	self deny: ref exists.
	self assert: deletions isEmpty.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self deny: (workingCopy root resolve: 'dir1/file2') exists.
	self deny: (objectMap includesKey: FSPath root / 'dir1' / 'file2').
	self assert: (#('/' '/file1' '/dir1') 
			includesAllOf: (fs root allChildren collect: [ :each | each path asString ]))! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 18:16'!
testDeletingWithReload
	| ref deletions objectMap |
	self createSomeCommits.
	deletions := fs instVarNamed: 'deletions'.
	objectMap := fs instVarNamed: 'objectMap'.
	ref := fs root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: deletions isEmpty.
	self assert: (objectMap includesKey: FSPath root / 'dir1' / 'file2').
	
	ref delete.
	self assert: deletions size = 1.
	self deny: ref exists.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self assert: (workingCopy root resolve: 'dir1/file2') exists.
	self assert: (objectMap includesKey: FSPath root / 'dir1' / 'file2'). 
	
	self commit: 'a commit'.
	fs := fsRepository head.
	ref := fs root resolve: 'copy'.
	modified := fs instVarNamed: 'modified'.
	workingCopy := fs instVarNamed: 'workingCopy'.
	deletions := fs instVarNamed: 'deletions'.
	self deny: ref exists.
	self assert: deletions isEmpty.
	self deny: (modified root resolve: 'dir1/file2') exists.
	self deny: (workingCopy root resolve: 'dir1/file2') exists.
	self deny: (objectMap includesKey: FSPath root / 'dir1' / 'file2').
	self assert: (#('/' '/file1' '/dir1' ) 
			includesAllOf: (fs root allChildren collect: [ :each | each path asString ]))! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:55'!
testLoadExistingFiles
	self createSomeCommits.
	fs := FSGitFilesystem for: basicRepository.
	fs root children do: [ :child |
		self assert: (#('/file1' '/dir1') includes: child path asString) ].
	fs root allChildren do: [ :child |
		self assert: (#('/' '/file1' '/dir1' '/dir1/file2') includes: child path asString) ]! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:55'!
testLoadExistingFilesFromHead
	self createSomeCommits.
	fs := FSGitFilesystem for: basicRepository.
	fs root children do: [ :child |
		self assert: (#('/file1' '/dir1') includes: child path asString) ].
	fs root allChildren do: [ :child |
		self assert: (#('/' '/file1' '/dir1' '/dir1/file2') includes: child path asString) ]! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:55'!
testLoadExistingFilesFromRevision
	self createSomeCommits.
	fs := FSGitFilesystem for: basicRepository revision: basicRepository head parents first.
	fs root children do: [ :child |
		self assert: (#('/file1') includes: child path asString) ].
	fs root allChildren do: [ :child |
		self assert: (#('/' '/file1') includes: child path asString) ]! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/16/2010 10:53'!
testModifyExistingFile
	| ref originalContent |
	self createSomeCommits.
	self assert: fs root children size = 2.
	self assert: fs root allChildren size = 4.
	
	ref := fs root resolve: 'dir1/file2'.
	originalContent := ref readStream contents asString.
	ref writeStreamDo: [ :stream | 
		stream 
			nextPutAll: originalContent;
			nextPutAll: '/moremoremore' ].
	self assert: fs root children size = 2.
	self assert: fs root allChildren size = 4.
	ref := workingCopy root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: ref readStream contents asString = 'more content'.
	ref := modified root resolve: 'dir1/file2'.
	self assert: ref exists.
	self assert: ref readStream contents asString = 'more content/moremoremore'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/24/2010 17:38'!
testModifying
	| ref |	
	self createSomeCommits.
	ref := fs root resolve: 'file1'.
	self assert: ref readStream contents asString = 'content'.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	
	self commit: 'a commit'.
	self assert: ref readStream contents asString = 'changed it'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/24/2010 17:38'!
testModifyingWithReload
	| ref |	
	self createSomeCommits.
	ref := fs root resolve: 'file1'.
	self assert: ref readStream contents asString = 'content'.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	
	self commit: 'a commit'.
	fs := fsRepository head.
	ref := fs root resolve: 'file1'.
	self assert: ref readStream contents asString = 'changed it'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:16'!
testParents
	fs parents isEmpty.
	
	(fs root / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	self assert: fs parents isEmpty.
	
	(fs root / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit number 2'.
	self deny: fs parents isEmpty.
	self assert: fs parents size = 1! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:16'!
testParentsWithReload
	fs parents isEmpty.
	
	(fs root / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	fs := fsRepository head.
	self assert: fs parents isEmpty.
	
	(fs root / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit number 2'.
	fs := fsRepository head.
	self deny: fs parents isEmpty.
	self assert: fs parents size = 1! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/16/2010 12:47'!
testReadExistingFile
	self createSomeCommits.
	self deny: (modified root resolve: 'file1') exists.
	self assert: (workingCopy root resolve: 'file1') exists.
	self assert: (fs root resolve: 'file1') readStream contents asString = 'content'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/16/2010 12:49'!
testReadExistingModifiedFile
	self createSomeCommits.
	(fs root resolve: 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	self assert: (fs root resolve: 'file1') readStream contents asString = 'changed it'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/14/2010 18:11'!
testReadNonexistentFile
	self 
		should: [ fs readStreamOn: 'missing File' ]
		raise: FSFileDoesNotExist! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:32'!
testReset
	| stream |
	self assert: modified root children isEmpty.
	self assert: workingCopy root children isEmpty.
	stream := fs writeStreamOn: 'afile'.
	[ stream nextPutAll: 'blah' ]
		ensure: [ stream close ].
	(fs root resolve: 'aFolder') ensureDirectory.
	stream := [ fs writeStreamOn: 'aFolder/afile' ]
		ensure: [ stream close ].
	self assert: modified root children size = 2.
	self assert: modified root allChildren size = 4.
	
	self reset.
	self assert: modified root children isEmpty.
	self assert: fs root children isEmpty! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 16:48'!
testResetWithDeletions
	| ref |
	self createSomeCommits.
	ref := fs root resolve: 'file1'.
	self assert: ref exists.
	
	ref delete.
	self deny: ref exists.
	
	self reset.
	self assert: ref exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:17'!
testRevisionChange
	| oldRevision newRevision |
	oldRevision := fs instVarNamed: 'revision'.
	self assert: oldRevision isNil.
	
	(fs root / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	oldRevision := fs instVarNamed: 'revision'.
	self deny: oldRevision isNil.
	
	(fs root / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	newRevision := fs instVarNamed: 'revision'.
	self deny: oldRevision = newRevision.
	self assert: (newRevision parents includes: oldRevision)! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:17'!
testRevisionChangeWithReload
	| oldRevision newRevision |
	oldRevision := fs instVarNamed: 'revision'.
	self assert: oldRevision isNil.
	
	(fs root / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	fs := fsRepository head.
	oldRevision := fs instVarNamed: 'revision'.
	self deny: oldRevision isNil.
	
	(fs root / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	fs := fsRepository head.
	newRevision := fs instVarNamed: 'revision'.
	self deny: oldRevision = newRevision.
	self assert: (newRevision parents includes: oldRevision)! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:17'!
testTagging
	| ref |
	(fs root / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit' andTag: 'my first tag'.
	ref := basicRepository tagsDir / 'my first tag'.
	self assert: ref exists.
	self assert: ref basename = 'my first tag'.
	self assert: (basicRepository objectNamed: ref readStream contents asString) message = 'a commit'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/5/2010 16:18'!
testTags
	self assert: fs tagNames isEmpty.
	
	(fs root / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1' andTag: 'tag 1'.
	self deny: fs tagNames isEmpty.
	self assert: fs tagNames size = 1.
	self assert: (fs tagNames includes: 'tag 1').
	
	(fs root / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2' andTag: 'tag number two'.
	self deny: fs tagNames isEmpty.
	self assert: fs tagNames size = 1.
	self assert: (fs tagNames includes: 'tag number two').
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag3' 
		to: (GitObjectSignature from: fs name).
	self assert: fs tagNames size = 2.
	self assert: (fs tagNames includesAllOf: #('tag number two' 'tag3'))! !

TestCase subclass: #FSGitLazyLoadingTests
	instanceVariableNames: 'filesystem repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitLazyLoadingTests methodsFor: 'as yet unclassified' stamp: 'MaxLeske 10/17/2010 22:16'!
setUp
	| workingCopy |
	filesystem := FSMemoryFilesystem new.
	repository := FSGitRepository on: filesystem root.
	workingCopy := repository head.

	workingCopy root / 'file1' writeStreamDo: [ :stream | stream nextPutAll: 'file1' ].
	workingCopy root / 'file2' writeStreamDo: [ :stream | stream nextPutAll: 'file2' ].
	workingCopy root / 'file3' writeStreamDo: [ :stream | stream nextPutAll: 'file3' ].
	workingCopy commit: 'added 3 files to root'.

	(workingCopy root / 'dir1') createDirectory.
	(workingCopy root resolve: 'dir1/file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir1' ].
	(workingCopy root resolve: 'dir1/file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir1' ].
	(workingCopy root resolve: 'dir1/file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir1' ].
	workingCopy commit: 'added a directory with 3 files to root'.
	
	(workingCopy root / 'dir2') createDirectory.
	(workingCopy root resolve: 'dir2/file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir2' ].
	(workingCopy root resolve: 'dir2/file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir2' ].
	(workingCopy root resolve: 'dir2/file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir2' ].
	workingCopy commit: 'added 3 files to dir2'.
	
	(workingCopy root / 'dir2' / 'dir2.1') createDirectory.
	(workingCopy root resolve: 'dir2/dir2.1/file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir2.1' ].
	(workingCopy root resolve: 'dir2/dir2.1/file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir2.1' ].
	(workingCopy root resolve: 'dir2/dir2.1/file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir2.1' ].
	workingCopy commit: 'added a subdir to dir2 and 3 files to the subdir'! !

!FSGitLazyLoadingTests methodsFor: 'as yet unclassified' stamp: 'MaxLeske 10/12/2010 22:23'!
testBasicCommit
	| repo root |
	repo := FSGitRepository on: (FSDiskFilesystem current working / 'testRepo').
	root := repo head root.
	
	self assert: (root / 'newFile') exists not.
	(root / 'newFile') writeStreamDo: [ :stream | stream nextPutAll: 'testcontent' ].
	root filesystem commit: 'test commit'.
	
	self assert: (repo head root / 'newFile') exists! !

!FSGitLazyLoadingTests methodsFor: 'as yet unclassified' stamp: 'MaxLeske 10/17/2010 19:52'!
testLazyLoading
	| entries |
	entries := #(
		'/'
		'file1' 'file2' 'file3'
		'dir1' 'dir1/file1' 'dir1/file2' 'dir1/file3'
		'dir2' 'dir2/file1' 'dir2/file2' 'dir2/file3'
		'dir2/dir2.1' 'dir2/dir2.1/file1' 'dir2/dir2.1/file2' 'dir2/dir2.1/file3').
	self assert: (repository head root allChildren includesAllOf: entries).
	self assert: ((repository head root / 'file1') readStreamDo: [ :stream | stream ]) = 'file1'! !

!FSGitLazyLoadingTests methodsFor: 'as yet unclassified' stamp: 'MaxLeske 10/12/2010 09:39'!
testLookup
	| repo root |
	repo := FSGitRepository on: FSDiskFilesystem current working / 'testRepo'.
	self assert: (repo head root / 'rb-spelling.dat') exists.
	self assert: (repo head root / 'rb-spelling.dat') isFile.
	self assert: (repo head root / 'package-cache') exists.
	self assert: (repo head root / 'package-cache') isDirectory.
	self assert: repo head root children size = 2.
	self assert: repo head root entries size = 2.
	self assert: repo head root allChildren size = 361.
	self assert: repo head root allEntries size = 361.
	self assert: repo head root childFiles size = 1.
	
	root := repo head root.
	self assert: (root / 'rb-spelling.dat') exists.
	self assert: (root / 'rb-spelling.dat') isFile.
	self assert: (root / 'package-cache') exists.
	self assert: (root / 'package-cache') isDirectory.
	self assert: root children size = 2.
	self assert: root entries size = 2.
	self assert: root allChildren size = 361.
	self assert: root allEntries size = 361.
	self assert: root childFiles size = 1! !

TestCase subclass: #FSGitRepositoryTests
	instanceVariableNames: 'repository basicRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitRepositoryTests methodsFor: 'running' stamp: 'MaxLeske 7/25/2010 16:12'!
setUp
	| repoRef repo commit1 commit2 tree blob |
	GitStamp
		defaultName: 'Homer Simpson';
		defaultEmail: 'homer@nuke.com'.
		
	repoRef := FSMemoryFilesystem new root resolve: 'testRepo'.
	repo := GitRepository on: repoRef.
	blob := GitBlob bytes: 'blob1'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'first blob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature }.
	commit1 := GitCommit 
		withTree: tree
		message: 'first commit'
		author: GitStamp now
		committer: GitStamp now.
	repo
		storeObject: blob;
		storeObject: tree;
		storeObject: commit1;
		updateRef: repo headsDir / 'branch1' to: commit1 signature.
	
	blob := GitBlob bytes: 'blob2'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'second blob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature }.
	commit2 := GitCommit
		withTree: tree
		message: 'second commit'
		author: GitStamp now
		committer: GitStamp now.
	commit2 parents: { commit1 }.
	repo
		storeObject: blob;
		storeObject: tree;
		storeObject: commit2;
		updateRef: repo headsDir / 'master' to: commit2 signature.
		
	basicRepository := repo.
	repository := FSGitRepository on: repoRef! !

!FSGitRepositoryTests methodsFor: 'running' stamp: 'MaxLeske 7/13/2010 10:10'!
tearDown
	GitStamp
		defaultName: nil;
		defaultEmail: nil! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:56'!
testBranches
	| branches |
	branches := repository branches.
	self assert: branches size = 2.
	branches do: [ :aBranch |
		self assert: aBranch species = FSGitFilesystem.
		self assert: ({ FSPath / 'first blob'. FSPath / 'second blob' } 
			includes: aBranch root children first path) ]! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:39'!
testFullTags
	| commit tags |
	commit := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/branch1').
	[ [ GitTag
		makeTagNamed: 'version 4.3alpha'
		for: commit
		message: 'PGP encrypted string'
		in: basicRepository ] 
			on: GitStampNameRequest 
			do: [ :req | req resume: 'Homer Simpson' ] ]
				on: GitStampEmailRequest
				do: [ :req | req resume: 'homer@nuke.com' ].
			
	tags := repository tags.
	self assert: tags size = 1.
	self assert: tags first species = FSGitFilesystem.
	self assert: (tags first instVarNamed: 'revision') = commit! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:56'!
testHead
	| head fsOnHead |
	head := basicRepository objectWithSignature: (basicRepository resolveRef: 'HEAD').
	fsOnHead := repository head.
	self assert: fsOnHead species = FSGitFilesystem.
	self assert: head = (fsOnHead instVarNamed: 'revision')! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 15:24'!
testLightTags
	| master branch1 tags |
	master := basicRepository resolveRef: 'refs/heads/master'.
	branch1 := basicRepository resolveRef: 'refs/heads/branch1'.
	basicRepository 
		updateRef: basicRepository tagsDir / 'version 4.3alpha' to: master;
		updateRef: basicRepository tagsDir / 'fail' to: branch1.
		
	tags := repository tags.
	self assert: tags size = 2.
	tags do: [ :each |
		self assert: each species = FSGitFilesystem.
		self assert: ({ master. branch1} includes: (each instVarNamed: 'revision') signature) ]! !

!FSGitRepositoryTests methodsFor: 'commit tests' stamp: 'MaxLeske 7/25/2010 16:05'!
testTags
	self assert: repository tags isEmpty.
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag1' 
		to: basicRepository head signature.
	self deny: repository tags isEmpty.
	self assert: repository tags size = 1.
	self assert: (repository tags includes: repository head).
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag2' 
		to: basicRepository head parents first signature.
	self deny: repository tags isEmpty.
	self assert: repository tags size = 2.
	self assert: (repository tags includes: repository head).
	self assert: (repository tags includes: repository head parents first)! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:56'!
testVersions
	| versions |
	versions := repository versions.
	versions do: [ :version |
		self assert: version species = FSGitFilesystem.
		self assert: ({ FSPath / 'first blob'. FSPath / 'second blob' } 
			includes: version root children first path) ]! !

TestCase subclass: #FSUsage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 17:24'!
testReferencing
	| mem ref fileRef |
	mem := FSMemoryFilesystem new.
	ref := (mem working / 'dir').
	ref createDirectory.
	self assert: ref exists.
	
	fileRef := (ref / 'file').
	self deny: fileRef exists.
	fileRef writeStreamDo: [:stream | stream nextPutAll: 'foo'].
	self assert: fileRef exists.
	
	self assert: (mem working / 'dir') exists.
	self assert: (mem working / 'dir' / 'file') exists.
	self deny: (mem working / 'dir/file') exists! !

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 17:37'!
testReferencingSubDirs
	| mem ref1 ref2 |
	mem := FSMemoryFilesystem new.
	ref1 := mem working / 'test' / 'dir'.
	ref2 := mem working / 'test' / 'dir'.
	self assert: ref1 = ref2.
	
	ref2 := mem working / 'test/dir'.
	self assert: ref1 species = ref2 species.
	self assert: ref1 filesystem = ref2 filesystem.
	self deny: ref1 path = ref2 path.
	self deny: ref1 = ref2! !

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 18:05'!
testReferencingWithResolve
	| mem ref1 ref2 |
	mem := FSMemoryFilesystem new.
	ref1 := mem working / 'test'.
	ref1 := mem working resolve: 'test/blah'.
	ref2 := mem working / 'test' / 'blah'	.
	self assert: ref1 = ref2! !

TestCase subclass: #GitBlobTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitBlobTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:28'!
testRetrievingBytes
	| blob |
	blob := GitBlob bytes: 'testBlob'.
	self assert: blob bytes = ('testBlob' asByteArray).
	
	blob := GitBlob bytes: 3242.
	self assert: blob bytes = (3242 asByteArray).
	
	blob := GitBlob bytes: #[3 42 45 3].
	self assert: blob bytes = #[3 42 45 3]! !

TestCase subclass: #GitCommitTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitCommitTests methodsFor: 'helpers' stamp: 'MaxLeske 8/6/2010 15:16'!
defaultStamp
	^ [ [ GitStamp now ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:18'!
testInitializeWithEverything
	| commit tree message author committer blob |
	blob := GitBlob bytes: 'aBlob'.
	message := 'a test message'.
	author := (GitStamp new
				email: 'mail@server.ext';
				name: 'Homer Simpson';
				timeStamp: TimeStamp now).
	committer := (GitStamp new
					email: 'homer@simpson.spr';
					name: 'Bart Simpson';
					timeStamp: TimeStamp now;
					yourself).
	tree := GitTree entries: {GitTreeEntry 
								entryName: 'filename'
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	
	commit := GitCommit 
				withTree: tree
				message: message
				author: author
				committer: committer.
	
	self assert: commit tree = tree.
	self assert: commit message = message.
	self assert: commit author = author.
	self assert: commit committer = committer! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:55'!
testPrintOn
	| message commit template |
	message := 'printOn test'.
	commit := GitCommit new.
	commit message: message.
	
	template := 'a GitCommit(''printOn test'')'.
	
	self assert: commit printString = template.! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:34'!
testRetrievingParents
	| commit collection parent1 parent2 parent3 |
	parent1 := GitCommit new.
	parent2 := GitCommit new.
	parent3 := GitCommit new.
	collection := OrderedCollection new.
	collection add: parent1; add: parent2; add: parent3.
	commit := GitCommit new.
	commit parents: collection.
	
	self assert: commit parents size = 3.
	self assert: commit parents first == parent1.
	self assert: commit parents second == parent2.
	self assert: commit parents third == parent3! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 8/6/2010 15:16'!
testWritingSameCommitTwice
	| commit repository tree blob treeEntry |
	blob := GitBlob bytes: 'ablob'.
	treeEntry :=
		GitTreeEntry 
			entryName: 'theblob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature.
	tree := GitTree entries: { treeEntry }.
	commit := 
		GitCommit 
		withTree: tree 
		message: 'a commit'
		author: self defaultStamp
		committer: self defaultStamp.
	repository := GitRepository on: (FSMemoryFilesystem new root).
	repository storeObject: commit.
	repository updateRef: repository headsDir / 'master' to: commit signature.
	
	self shouldnt: [ repository storeObject: commit ] raise: Exception.! !

TestCase subclass: #GitObjectLoaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/27/2010 20:28'!
testFor
	| repository loader |
	repository := GitRepository on: FSMemoryFilesystem new root.
	loader := GitObjectLoader for: repository.
	self assert: (loader instVarNamed: 'repository')= repository! !

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/27/2010 20:25'!
testNew
	self should: [GitObjectLoader new] raise: Error! !

TestCase subclass: #GitObjectSignatureTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:54'!
authorEmail
	^'homer@nuke.com'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:55'!
authorName
	^'Homer Simpson'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/23/2010 18:56'!
authorTimeStamp
	^((TimeStamp fromUnixTime: 1274888648) + (Duration hours: 2)) offset: (Duration hours: 2)! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:18'!
blobContents
	^ 'aString' asByteArray! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:28'!
blobSignature
	^ GitObjectSignature for: self createBlob! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:26'!
blobTypeCode
	^ GitBlob typeCode! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:50'!
commitContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream 
		nextPutAll: self treeTypeCode asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self treeSignature hexHash asByteArray;
		nextPut: Character lf asInteger;
		"this commit has no parents"
		nextPutAll: 'author' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'committer' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPut: Character lf asInteger;
		nextPutAll: self commitMessage asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:49'!
commitMessage
	^ 'the message'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:32'!
commitSignature
	^ GitObjectSignature for: self createCommit! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
commitTypeCode
	^ GitCommit typeCode! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:18'!
createBlob
	^ GitBlob bytes: self blobContents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:49'!
createCommit
		^ GitCommit 
			withTree: self createTree
			message: self commitMessage
			author: self defaultStamp
			committer: self defaultStamp.! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/24/2010 17:18'!
createTag
	^ GitTag new
		name: self tagName;
		message: self tagMessage;
		tagger: self defaultStamp;
		object: self createCommit;
		yourself! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:55'!
createTree
	^ GitTree entries: { self createTreeEntry }! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:56'!
createTreeEntry
	^ GitTreeEntry
			entryName: self treeEntryName
			mode: GitTreeEntry fileMode
			objectSignature: self createBlob signature! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:35'!
defaultStamp
	| fixedTime |
	fixedTime := DateAndTime 
		year: 2010
		month: 7 
		day: 22
		hour: 19
		minute: 34
		second: 23
		offset: (Duration hours: 5).
		
	^ [ [ (GitStamp now) timeStamp: fixedTime ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/24/2010 17:18'!
referenceAuthor
	^ GitStamp new
		name: self authorName;
		email: self authorEmail;
		timeStamp: self authorTimeStamp;
		yourself! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:45'!
referenceBlobHash
	^'e7cbb71a0d5b66bd7116563b395a98a81e50f988'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceBlobSize
	^8! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:36'!
referenceCommitHash
	^'0f02985452ad9beebb2af39649145fbf5d95aa31'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceContent
	^'testfile'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceFileName
	^'TESTFILE.txt'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:11'!
referenceMessage
	^'testfile commit'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 6/23/2010 19:59'!
referenceTreeEntryMode
	^8r100644! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:52'!
referenceTreeHash
	^'ad8473da921cc7878fe1106d3083b477d6592de4'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 11:43'!
referenceTreeSize
	^40! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:20'!
tagContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream
		nextPutAll: 'object' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self commitSignature hexHash asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'type' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: GitCommit typeCode asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'tag' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self tagName asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'tagger' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPut: Character lf asInteger;
		nextPutAll: self tagMessage asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:13'!
tagMessage
	^ 'Maybe a PGP encrypted message'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:12'!
tagName
	^ 'some releaseRC1'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:16'!
tagSignature
	^ GitObjectSignature for: self createTag! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
tagTypeCode
	^ GitTag typeCode! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 7/31/2010 17:50'!
testBytePadding
	| signature testHash savedHash stream |
	testHash := '00a350a77157772e1e35bb27b0582a8c51094fac'.
	self assert: testHash size = 40.
	signature := GitObjectSignature from: testHash.
	self assert: signature hexHash = testHash.
	
	stream := GitStream new.
	signature saveTo: stream.
	savedHash := stream contents hex.
	self assert: savedHash size = 40.
	self assert: (savedHash beginsWith: '00a35').
	self deny: (savedHash beginsWith: 'a35').
	self assert: savedHash = testHash! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:25'!
testByteSignatureCreation
	"reference commit created with GitX 0.7.1 on git 1.7.0"
	| fileRef stream testCommit tree blob |	
	fileRef := FSMemoryFilesystem new root resolve: self referenceFileName.
	fileRef writeStreamDo: [:writer | writer nextPutAll: self referenceContent].
	stream := fileRef readStream.
	self assert:  stream contents size = self referenceBlobSize.	
	blob := GitBlob bytes: stream contents.
	
	tree := GitTree entries: {GitTreeEntry
								entryName: self referenceFileName
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	testCommit := GitCommit 
					withTree: tree
					message: self referenceMessage
					author: self referenceAuthor
					committer: self referenceAuthor.
	
	self assert: blob signature hexHash = self referenceBlobHash.
	self assert: GitTreeEntry fileMode  = self referenceTreeEntryMode.
	self assert: tree size = self referenceTreeSize.
	self assert: tree signature hexHash = self referenceTreeHash.
	self assert: testCommit signature hexHash = self referenceCommitHash! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/9/2010 11:42'!
testByteSignatureForBlob
	| stream bytes |
	stream := ReadStream on: (GitObjectSignature new byteSignatureFor: (GitBlob bytes: 'test')).
	
	bytes := stream upTo: Character space asInteger.
	self assert: bytes asString = 'blob'.
	
	bytes := stream upTo: 0.
	self assert: bytes asString asInteger = 4.
	
	bytes := stream upToEnd.
	self assert: bytes asString = 'test'! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 12:33'!
testEquals
	| signature1 signature2 |
	signature1 := GitObjectSignature for: (GitBlob bytes: 'test1').
	signature2 := GitObjectSignature for: (GitBlob bytes: 'test2').
	self deny: signature1 = signature2! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 7/22/2010 20:30'!
testHashingConditions
	| objectSignature objectContent objectTypeCode templateStream objectNames |
	objectNames := #('blob' 'commit' 'tree' 'tag').
	
	objectNames do:  [ :objectName |
		templateStream := WriteStream on: (ByteArray new).
		objectTypeCode := self perform: (objectName , 'TypeCode') asSymbol.
		objectContent := self perform: (objectName , 'Contents') asSymbol.
		objectSignature := self perform: (objectName , 'Signature') asSymbol.
		
		templateStream 
			nextPutAll: objectTypeCode asByteArray;
			nextPut: Character space asInteger;
			nextPutAll: objectContent size asString asByteArray;
			nextPut: 0;
			nextPutAll: objectContent.
		
		self assert: templateStream contents = objectSignature bytes ]! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/9/2010 11:44'!
testHexHashFromIntegerHash
	| hash |
	hash := SecureHashAlgorithm new hashMessage: 11.
	self assert: hash hex size = 39.
	hash := GitObjectSignature new hexHashFrom: hash.
	self assert: hash size = 40.
	self assert: hash asLowercase = hash.! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:48'!
testPrintOn
	| signature template |
	signature := GitObjectSignature from: 'bf8b4530d8d246dd74ac53a13471bba17941dff7'.
	template := 'a GitObjectSignature(bf8b4530d8d246dd74ac53a13471bba17941dff7)'.
	
	self assert: signature printString = template! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:02'!
treeContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream
		nextPutAll: (GitTreeEntry fileMode printStringBase: 8) asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self treeEntryName asByteArray;
		nextPut: 0;
		nextPutAll: self createBlob signature integerHash asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:55'!
treeEntryName
	^ 'a filename'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:42'!
treeSignature
	^ GitObjectSignature for: self createTree! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
treeTypeCode
	^ GitTree typeCode! !

TestCase subclass: #GitRepositoryTests
	instanceVariableNames: 'repository reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitRepositoryTests methodsFor: 'helpers' stamp: 'MaxLeske 7/13/2010 16:04'!
defaultStamp
	^ [ [ GitStamp now ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitRepositoryTests methodsFor: 'running' stamp: 'MaxLeske 7/25/2010 16:07'!
setUp
	| commit1 commit2  blob1 blob2 tree1 tree2 stamp |
	stamp := GitStamp new
		name: 'Homer Simpson';
		email: 'homer@nuke.com';
		yourself.
	reference := (FSMemoryFilesystem new root resolve: 'test.git').
	repository := GitRepository on: reference.

	blob1 := GitBlob bytes: 'testBlob'.
	blob2 := GitBlob bytes: 'second test Blob'.
	tree1 := GitTree entries: {
		GitTreeEntry
			entryName: 'blob1'
			mode: GitTreeEntry fileMode
			objectSignature: blob1 signature}.
	tree2 := GitTree entries: {
		GitTreeEntry
			entryName: 'blob2'
			mode: GitTreeEntry fileMode
			objectSignature: blob2 signature}.
			
	commit1 := GitCommit 
		withTree: tree1
		message: 'message1'
		author: stamp 
		committer: stamp.
	commit2 := GitCommit
		withTree: tree2
		message: 'message2'
		author: stamp
		committer: stamp.
	commit2 parents: { commit1 }.
	
	repository
		storeObject: blob1;
		storeObject: blob2;
		storeObject: tree1;
		storeObject: tree2;
		storeObject: commit1;
		storeObject: commit2;
		updateRef: repository headsDir / 'branch1' to: commit1 signature;
		updateRef: repository headsDir / 'master' to: commit2 signature;
		updateRef: repository headFile to: commit2 signature! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:07'!
testBaseDir
	self assert: (reference basename endsWith: '.git').
	self assert: repository baseDir = reference.
	
	reference := reference resolve: 'noExtension'.
	repository := GitRepository on: reference.
	self deny: repository baseDir = reference.
	self assert: repository baseDir = (reference resolve: '.git')! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/11/2010 13:05'!
testBranches
	| branches |
	branches := repository branches.
	self assert: branches size = 2.
	branches do: [ :each |
		self assert: (each isKindOf: GitCommit).
		self assert: (#('message1' 'message2') includes: each message)]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 15:19'!
testCreateReferencedDirectory
	| filesystem |
	filesystem := FSMemoryFilesystem new.
	reference := filesystem root resolve: 'alpha/beta/gamma/repo.git'.
	repository := GitRepository on: reference.
	
	repository createReferencedDirectory.
	filesystem root allChildren do: [:each | self assert: each exists].
	
	filesystem := FSMemoryFilesystem new.
	reference := filesystem root resolve: '/alpha/beta/repo.git'.
	(filesystem root / 'alpha') createDirectory.
	repository := GitRepository on: reference.
	
	self 
		shouldnt: [repository createReferencedDirectory]
		raise: FSDirectoryExists
		description: 'The code should check if one of parent directories exists before trying to create it'.
	self assert: (filesystem root / 'alpha') exists.
	self assert: (filesystem root / 'alpha' / 'beta') exists.
	self assert: (filesystem root / 'alpha' / 'beta' / 'repo.git') exists.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 15:17'!
testDelete
	self assert: repository exists description: 'Need a repository to delete.'.
	[ repository delete ]
		on: GitRepositoryDeleteRequest 
		do: [ :req | req resume: false].
	self assert: repository exists.
	
	[ repository delete ]
		on: GitRepositoryDeleteRequest 
		do: [ :req | req resume: true].
	self deny: repository exists description: 'Deletion of repository failed.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:00'!
testDirDoesntExist
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: (reference resolve: 'newRepo').
	self deny: repository exists.
	repository gitInitialize.
	self assert: repository exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:02'!
testDirExistsButNotInitialized
	reference := reference resolve: 'newRepo'.
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: reference.
	reference createDirectory.
	self deny: repository exists.
	repository gitInitialize.
	self assert: repository exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:17'!
testEquals
	| repo2 |
	repo2 := GitRepository on: FSMemoryFilesystem new working.
	self deny: repository = repo2.
	
	repo2 := GitRepository on: repository baseDir.
	self assert: repository = repo2! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:03'!
testGitInitializeOnReferenceWithSubDirs
	reference := FSMemoryFilesystem new root resolve: 'alpha/beta/testRepo.git'.
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: reference.
	self 
		shouldnt: [repository gitInitialize]
		raise: FSDirectoryDoesNotExist
		description: 'The reference of the repository points to a directory'
			, 'inside of a parent directory that does not exist.'
			, 'Create the parent directory first.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/1/2010 10:58'!
testInitializeOnDiskFilesystem
	reference := FSDiskFilesystem current working resolve: 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository baseDir filesystem isKindOf: FSDiskFilesystem).
	self assert: repository baseDir = (reference / '.git').
	reference deleteAll.
	self deny: reference exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:22'!
testInitializeOnMemoryFilesystem
	reference := FSMemoryFilesystem new working resolve: 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository baseDir filesystem isKindOf: FSMemoryFilesystem).
	self assert: repository baseDir = (reference / '.git')! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:04'!
testIsInitialized
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: (FSMemoryFilesystem new root resolve: 'test.git').
	self deny: repository isInitialized.
	
	repository gitInitialize.
	self assert: repository isInitialized! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 4/28/2010 19:44'!
testNew
	self should: [GitRepository new] raise: Error! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:39'!
testObjectWithSignature
	| initialBlob loadedBlob |
	initialBlob := GitBlob bytes: 'test'.
	repository storeObject: initialBlob.
	loadedBlob := repository objectWithSignature: initialBlob signature.
	self assert: initialBlob = loadedBlob! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 4/28/2010 19:44'!
testOnClass
	self should: [GitRepository on: FSDiskFilesystem]
		raise: Error
		description: 'Please provide an instance of a filesystem, not a class.'.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 15:27'!
testResolveRef
	| blob resolved |
	blob := repository storeObject: (GitBlob bytes: 'test').
	repository updateRef: 'HEAD' to: blob signature.
	
	resolved := repository resolveRef: 'HEAD'.
	self assert: blob signature = resolved! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:20'!
testSetupDirectoryStructure
	| dirs |
	dirs := #(
		'HEAD'
		'info'
		'objects'
		'objects/pack'
		'objects/info'
		'refs'
		'refs/heads'
		'refs/tags').
	dirs do: [ :dir | self assert: (repository baseDir resolveString: dir) exists ]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:07'!
testStoreObjectBlob
	| blob storedObject signature hash |
	blob := GitBlob bytes: 'test'.
	signature := blob signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: blob.
	self assert: storedObject = blob! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:06'!
testStoreObjectCommit
	| tree commit signature hash storedObject |
	tree := GitTree new.
	commit := GitCommit new.
	commit
		author: self defaultStamp;
		committer: self defaultStamp;
		tree: tree;
		message: 'testCommit'.
	signature := commit signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: commit.
	self assert: storedObject = commit! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/26/2010 21:51'!
testStoreObjectCommitNoAuthor
	| commit |
	commit := GitCommit new.
	commit tree: GitTree new.
	self 
		should: [repository storeObject: commit] 
		raise: Error 
		whoseDescriptionIncludes: 'Author identity is required.'
		description: 'Author needs to be set before storing the commit.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 4/29/2010 18:43'!
testStoreObjectCommitNoTree
	| commit |
	commit := GitCommit new.
	self 
		should: [repository storeObject: commit] 
		raise: Error 
		whoseDescriptionIncludes: 'Cannot have a commit without a tree.'
		description: 'Every commit needs a tree. Make sure this is checked for.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/8/2010 17:05'!
testStoreObjectTree
	| tree storedObject |
	tree := GitTree new.
	storedObject := repository storeObject: tree.
	self assert: storedObject = tree! !

TestCase subclass: #GitStampTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitStampTests methodsFor: 'helpers' stamp: 'MaxLeske 7/13/2010 16:25'!
defaultStamp
	^ [ [ GitStamp now ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitStampTests methodsFor: 'tests' stamp: 'MaxLeske 7/13/2010 16:02'!
testNow
	| stamp now |
	stamp := self defaultStamp.
	now := TimeStamp now.
	
	self assert: stamp timeStamp <= now.	
	self assert: stamp name = 'Homer Simpson'.
	self assert: stamp email = 'homer@nuke.com'! !

!GitStampTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 17:39'!
testPrintOn
	| stamp template now stream |
	now := TimeStamp now.
	stamp := self defaultStamp.
	stamp timeStamp: now.
	
	template := 
		'Homer Simpson <homer@nuke.com> ', 
		now asUnixTime asString, 
		' ',
		(now offset positive ifTrue: ['+'] ifFalse: ['-']),
		(now offset hours printString forceTo: 2 paddingStartWith: $0),
		(now offset minutes printString forceTo: 2 paddingStartWith: $0).
	
	self assert: stamp printString = template! !

!GitStampTests methodsFor: 'tests' stamp: 'MaxLeske 7/22/2010 18:48'!
testStampReading
	| str stamp |
	str := 'Tony Garnock-Jones <tonygarnockjones@gmail.com> 1266614121 +1300'.
	stamp := GitStamp fromString: str.
	self assert: str = stamp printString.! !

TestCase subclass: #GitStorableObjectTests
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitStorableObjectTests class methodsFor: 'package selection' stamp: 'MaxLeske 7/24/2010 17:26'!
packageNamesUnderTest
	^#('FS-Git-Plumbing')! !

!GitStorableObjectTests methodsFor: 'helpers' stamp: 'MaxLeske 7/26/2010 09:01'!
buildMockFor: aGitObjectClass
	| tree fakeName |
	fakeName := GitObjectSignature for: (GitBlob bytes: 'test').
	tree := GitTree entries: {
		(GitTreeEntry 
			entryName: 'testFile' 
			mode: GitTreeEntry fileMode 
			objectSignature: fakeName)}.
	aGitObjectClass = GitBlob
		ifTrue: [ ^ GitBlob bytes: 'test' ].
	aGitObjectClass = GitCommit
		ifTrue: [ 
			^ (GitCommit new)
				author: self defaultStamp;
				committer: self defaultStamp;
				tree: tree;
				message: 'initial commit';
				yourself ].
	aGitObjectClass = GitTag
		ifTrue: [ 
			^ GitTag
				makeTagNamed: 'testTag'
				for: (repository storeObject: (self buildMockFor: GitCommit))
				message: 'aMessage'
				tagger: self defaultStamp
				in: repository ].
	aGitObjectClass = GitTree
		ifTrue: [ ^ GitTree entries: {(GitTreeEntry entryName: 'testFile' mode: GitTreeEntry fileMode objectSignature: fakeName)} ].
	self error: 'the class you provided needs to be instantieted to test #loadFrom'! !

!GitStorableObjectTests methodsFor: 'helpers' stamp: 'MaxLeske 7/13/2010 16:00'!
defaultStamp
	^ [ [ GitStamp now ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitStorableObjectTests methodsFor: 'running' stamp: 'MaxLeske 7/26/2010 09:01'!
setUp
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'repo')! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/24/2010 14:46'!
testForType
	GitStorableObject allSubclasses do: [:each | each typeCode isNil 
		ifFalse: [self assert: (GitStorableObject forType: each typeCode) = each]
		ifTrue: [self should: [GitStorableObject forType: each typeCode] raise: Error]]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 7/26/2010 09:00'!
testLoadFrom
	| stream object loadedObject clazzes |
	clazzes := GitStorableObject allSubclasses reject: [:each | each typeCode isNil].
	clazzes do:
		[:each | object := self buildMockFor: each.
		repository storeObject: object.
		stream := repository looseObjectStreamAt: object signature.
		loadedObject := GitStorableObject loadFrom: stream from: repository. 
		self assert: loadedObject isNil not.
		self assert: loadedObject species = (object species).
		self assert: loadedObject = object]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/24/2010 14:46'!
testTypeCode
	self assert: GitStorableObject typeCode isNil! !

TestCase subclass: #GitTagTests
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTagTests methodsFor: 'running' stamp: 'MaxLeske 7/13/2010 16:05'!
setUp
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'testRepo')! !

!GitTagTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 18:23'!
testMakeLightTag
	| commit tree blob |
	blob := GitBlob bytes: 'test'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'testFile'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature}.
	commit := [ [ GitCommit withDefaultStamps ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ].
	commit
		tree: tree;
		message: 'myCommit'.
	
	GitTag makeLightTagNamed: 'testTag' for: commit in: repository.	
	self assert: (repository tagsDir / 'testTag') exists! !

TestCase subclass: #GitTaglikeObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 20:48'!
testLoadingAssociations
	| taglikeObject |
	taglikeObject := GitTaglikeObject new.
	self assert: taglikeObject properties isEmpty.
	
	taglikeObject loadKey: 'keyname' value: 'keyobject'.
	self assert: taglikeObject properties first species = Association.
	self assert: taglikeObject properties first key = 'keyname'
	! !

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 20:44'!
testRetrievingProperties
	| object collection |
	self assert: GitTaglikeObject new properties isEmpty.
	
	collection := OrderedCollection new.
	collection add: Object new.
	object := GitTaglikeObject new.
	object properties: collection.
	
	self assert: object properties = collection.
	self assert: object properties isEmpty not! !

TestCase subclass: #GitTreeEntryTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTreeEntryTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:46'!
testLessThanOrEqual
	| entry1 entry2 entry3 entry4 entry5 |
	entry1 := GitTreeEntry 
		entryName: 'Connencton.doc'
		mode: nil
		objectSignature: nil.
	entry2 := GitTreeEntry 
		entryName: 'GitFS note.txt'
		mode: nil
		objectSignature: nil.
	entry3 := GitTreeEntry
		entryName: 'dummy.docx'
		mode: nil
		objectSignature: nil.
	entry4 := GitTreeEntry
		entryName: 'aFolder'
		mode: GitTreeEntry dirMode
		objectSignature: nil.
	entry5 := GitTreeEntry
		entryName: 'aFolder'
		mode: GitTreeEntry fileMode
		objectSignature: nil.
	
	self assert: entry1 <= entry2.
	self assert: entry1 <= entry3.
	self assert: entry2 <= entry3. "asciiValue of $G is smaller than that of $d"
	self assert: entry5 <= entry4 "entries for tree objects are compared with a trailing slash"! !

!GitTreeEntryTests methodsFor: 'class-tests' stamp: 'MaxLeske 7/10/2010 11:18'!
testPrintMode
	self assert: (GitTreeEntry printMode: GitTreeEntry fileMode) = '100644'.
	self assert: (GitTreeEntry printMode: GitTreeEntry dirMode) = '40000'! !

!GitTreeEntryTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testSize
	| treeEntry fileName mode signature sizeToBe |
	fileName := 'testFile'.
	mode := GitTreeEntry printMode: GitTreeEntry fileMode.
	signature := (GitBlob bytes: 'fooBlob') signature.
	treeEntry := GitTreeEntry 
					entryName: fileName 
					mode: GitTreeEntry fileMode
					objectSignature: signature.
					
	sizeToBe := 
		(mode size) +
		(Character space asInteger asByteArray size) +
		(fileName size) +
		(0 decimalDigitLength) +
		(signature size).
	
	self assert: signature size = 20.
	self assert: treeEntry size = sizeToBe.! !

TestCase subclass: #GitTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testAt
	| tree entry blob repo |
	blob := GitBlob bytes: 'testblob'.
	repo := GitRepository on: (FSMemoryFilesystem new root resolve: 'testRepo').
	repo storeObject: blob.
	entry := GitTreeEntry
		entryName: 'testFile'
		mode: GitTreeEntry fileMode
		objectSignature: blob signature.
	tree := GitTree in: repo.
	tree entries: {entry}.
	
	self assert: (tree at: entry entryName) = blob! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/26/2010 22:30'!
testNoEntries
	self assert: (GitTree new entries) species = Dictionary.
	self assert: (GitTree new entries) isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testRemoveEntry
	| tree collection entry1 entry2 entry3 |
	entry1 := GitTreeEntry
		entryName: 'entry 1'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 1).
	entry2 := GitTreeEntry
		entryName: 'entry 2'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 2).
	entry3 := GitTreeEntry
		entryName: 'entry 3'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 3).
	collection := OrderedCollection new add: entry1; add: entry2; add: entry3; yourself.
	tree := GitTree entries: collection.
	
	self assert: tree entries size = 3.
	self assert: (tree entries includesKey: entry1 entryName).
	self assert: (tree entries includesKey: entry2 entryName).
	self assert: (tree entries includesKey: entry3 entryName).
	
	tree removeEntryAt: entry2 entryName.
	self assert: tree entries size = 2.
	self deny: (tree entries includesKey: entry2 entryName).
	
	tree removeEntryAt: entry3 entryName.
	self assert: tree entries size = 1.
	self deny: (tree entries includesKey: entry3 entryName).
	
	tree removeEntryAt: entry1 entryName.
	self assert: tree entries isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testSettingEntries
	| collection tree entry1 entry2 |
	collection := OrderedCollection new.
	entry1 := GitTreeEntry 
		entryName: 'object1' 
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 234).
	entry2 := GitTreeEntry 
		entryName: 'object2' 
		mode: GitTreeEntry dirMode
		objectSignature: (GitObjectSignature from: 564).
	collection add: entry1; add: entry2.
	tree := GitTree entries: collection.
	
	self assert: (tree entries at: entry1 entryName)= entry1.
	self assert: (tree entries at: entry2 entryName) = entry2.
	self deny: entry1 = entry2! !

Object subclass: #FSGitModificationManager
	instanceVariableNames: 'createdPaths deletedPaths objectMap repository cache root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 18:03'!
cache: aFilesystem
	cache := aFilesystem! !

!FSGitModificationManager methodsFor: 'enumerating' stamp: 'MaxLeske 10/17/2010 18:02'!
createdPathsDo: aBlock	
	createdPaths associations do: [ :assoc | 
		assoc value do: [ :path |
			aBlock 
			value: assoc key 
			value: path ] ]! !

!FSGitModificationManager methodsFor: 'testing' stamp: 'MaxLeske 10/17/2010 17:30'!
hasModifications
	^ createdPaths isEmpty not or: [ deletedPaths isEmpty not ]! !

!FSGitModificationManager methodsFor: 'initialize-release' stamp: 'MaxLeske 10/17/2010 17:26'!
initialize
	super initialize.
	objectMap := IdentityDictionary new.
	createdPaths := IdentityDictionary new.
	deletedPaths := IdentityDictionary new! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 21:46'!
markCreated: aPath inTree: aTree
	aPath parent = FSPath root ifTrue: [ root := aTree].
	createdPaths
		at: aTree
		ifPresent: [ :paths | paths add: aPath ]
		ifAbsent: [ | paths |
			paths := Set new.
			paths add: aPath.
			createdPaths
				at: aTree
				put: paths ]! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 14:14'!
markDeleted: aPath inTree: aTree
	deletedPaths
		at: aTree
		ifPresent: [ :paths | paths add: aPath ]
		ifAbsent: [ | paths |
			paths := Array new.
			paths add: aPath.
			deletedPaths
				at: aTree
				put: paths ]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'MaxLeske 10/17/2010 19:45'!
newTreeAt: aPath
	^ GitTree new! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'MaxLeske 10/17/2010 19:46'!
processBlobs
	| reference data blob entry |
	self createdPathsDo: [ :tree : path |
		reference := cache referenceTo: path.
		reference isFile ifTrue: [
			data := reference readStreamDo: [ :stream | stream contents ].
			blob := GitBlob bytes: data.
			repository storeObject: blob .
			
			entry := GitTreeEntry
				entryName: path basename
				mode: GitTreeEntry fileMode
				objectSignature: blob signature.
			tree putEntry: entry ] ]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'MaxLeske 10/17/2010 17:11'!
processDeletions
	self removeDeletedPathsFromTrees! !

!FSGitModificationManager methodsFor: 'private' stamp: 'MaxLeske 10/17/2010 14:19'!
removeDeletedPathsFromTrees
	deletedPaths associations do: [ :assoc |
		assoc value do: [ :path |
			assoc key removeEntryAt: path basename ] ]! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 15:01'!
repository: aRepository
	repository := aRepository ! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 18:19'!
root
	^ root! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'MaxLeske 10/17/2010 17:10'!
storeTrees
	createdPaths keys do: [ :tree |
		repository storeObject: tree ]! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 22:25'!
treeAt: aPath
	| tree |
	tree := self root
		ifNil: [ repository head 
			ifNil: [ self newTreeAt: aPath ]
			ifNotNil: [ root := repository head tree. root ] ].
	aPath do: [ :segment |
		tree := tree
			entryAt: segment
			ifAbsent: [ self newTreeAt: aPath ] ].
	^ tree! !

Object subclass: #FSGitRepository
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitRepository commentStamp: '<historical>' prior: 0!
I represent a git repository. Every message sent to me will return an FSGitFilesystem object or a collection of such objects.!

!FSGitRepository class methodsFor: 'instance creation' stamp: 'MaxLeske 7/9/2010 15:10'!
on: anFSReference
	^ self new initializeOn: anFSReference! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/28/2010 12:19'!
branches
	| dictionary |
	dictionary := repository branches.
	dictionary copy associationsDo: [ :assoc | 
		dictionary 
			at: assoc key 
			put: (FSGitFilesystem for: repository revision: assoc value) ].
		
	^ dictionary! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/21/2010 22:16'!
head
	^ FSGitFilesystem for: repository! !

!FSGitRepository methodsFor: 'initialize-release' stamp: 'MaxLeske 7/23/2010 09:59'!
initializeOn: anFSReference
	repository := GitRepository on: anFSReference! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/21/2010 22:17'!
tags
	^ repository tags collect: [ :tag | FSGitFilesystem for: repository revision: tag object ]! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/21/2010 22:18'!
versions
	| current commits |
	commits := OrderedCollection new.
	repository looseObjectNamesDo: [ :each | 
		current := repository objectWithSignature: each.
		current species = GitCommit
			ifTrue: [ commits add: current ] ].
	
	^ commits collect: [ :each |
		FSGitFilesystem for: repository revision: each ] ! !

Object subclass: #GitDeltaInterpreter
	instanceVariableNames: 'source instructions output commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions
	^ instructions! !

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions: anObject
	instructions := anObject! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:09'!
readVariableInteger
	| val shift b |
	val := 0.
	shift := 0.
	[
		b := commands next.
		val := val bitOr: ((b bitAnd: 127) bitShift: shift).
		shift := shift + 7.
	] doWhileFalse: [(b bitAnd: 128) isZero].
	^ val! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:20'!
run
	[commands atEnd] whileFalse: [self runNextCommand]! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:25'!
runAgainst: aByteArray
	source := aByteArray.
	
	commands := ReadStream on: instructions.
	source size = self readVariableInteger ifFalse: [self error: 'Delta source size mismatch'].
	output := WriteStream on: (ByteArray new: self readVariableInteger).
	self run.

	^ output contents! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:19'!
runNextCommand
	| cmd |
	cmd := commands next.
	cmd = 0 ifTrue: [self error: 'unexpected delta opcode 0'].
	cmd < 128
		ifTrue: [output nextPutAll: (commands next: cmd)]
		ifFalse: [
			| off size |

			off := 0.
			(cmd bitAnd: 2r00000001) isZero ifFalse: [off := commands next].
			(cmd bitAnd: 2r00000010) isZero ifFalse: [off := off bitOr: (commands next bitShift: 8)].
			(cmd bitAnd: 2r00000100) isZero ifFalse: [off := off bitOr: (commands next bitShift: 16)].
			(cmd bitAnd: 2r00001000) isZero ifFalse: [off := off bitOr: (commands next bitShift: 24)].

			size := 0.
			(cmd bitAnd: 2r00010000) isZero ifFalse: [size := commands next].
			(cmd bitAnd: 2r00100000) isZero ifFalse: [size := size bitOr: (commands next bitShift: 8)].
			(cmd bitAnd: 2r01000000) isZero ifFalse: [size := size bitOr: (commands next bitShift: 16)].
			size = 0 ifTrue: [size := 16r10000].
			
			output nextPutAll: (source copyFrom: off + 1 to: off + size)]! !

Object subclass: #GitObject
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObject commentStamp: '<historical>' prior: 0!
I am an abstract object that defines messages to which every object (in git speak) responds.

Instance Variables:
	repository <GitRepository>
		A private variable that a GitObject uses to lazily load itself!

!GitObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/30/2010 14:12'!
in: aGitRepository
	^self new initializeOn: aGitRepository; yourself! !

!GitObject methodsFor: 'initialize-release' stamp: 'MaxLeske 7/24/2010 17:21'!
initializeOn: aGitRepository
	repository := aGitRepository! !

!GitObject methodsFor: 'initialize-release' stamp: 'MaxLeske 7/23/2010 14:52'!
loadFrom: aStream
"This method implements the loading of a GitObject from a stored byte sequence."

	self subclassResponsibility! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:24'!
repository
	^repository! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:24'!
repository: aGitRepository
	repository := aGitRepository! !

!GitObject methodsFor: 'saving' stamp: 'MaxLeske 7/23/2010 14:50'!
saveTo: aStream
"This method defines the byte sequence for the signature calculation (GitObjectSignature) and also the content of a particular object that is stored in the repository."

	self subclassResponsibility! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 14:47'!
size
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream size! !

GitObject subclass: #GitObjectSignature
	instanceVariableNames: 'signature integerHash'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObjectSignature commentStamp: '<historical>' prior: 0!
I know how to calculate a signature for each GitStorableObject and I handle the hash representation of that signature. My equivalent in git speak is the sha1 name which is a 40 character hash (SHA1 encryption of header and contents of each object; see http://git.rsbx.net/Documents/Git_Data_Formats.txt and http://book.git-scm.com/1_the_git_object_model.html) stored as a string.

Instance Variables:
	signature	<ByteArray>
		The byte sequence representing the signature for an object that is used to calculate it sha1 hash.
	integerHash	<Integer>
		The hash calculated from the signature byte sequence in integer representation.!

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 4/29/2010 18:05'!
for: aGitObject
	^self new computeSignatureFor: aGitObject! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/9/2010 13:13'!
from: aHexHash
	^self new from: aHexHash! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 7/9/2010 17:28'!
loadFrom: aStream
	| hash |
	hash := (aStream next: 20).
	^self from: hash hex! !

!GitObjectSignature methodsFor: 'comparing' stamp: 'MaxLeske 5/9/2010 15:15'!
= other
	^self integerHash = other integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 6/23/2010 19:58'!
byteSignatureFor: aGitObject
	| stream |
	stream := GitStream new.
	"format for object hash (see manual -> 'Object storage format'):
	<ascii type without space> + <space> + <ascii decimal size> + <byte\0> + <binary object data>"
	stream
		nextPutAll: aGitObject typeCode;
		space;
		nextPutAll: aGitObject byteData size asString; "ASCII value for $0 is not 0 but 48!!"
		nullByte;
		nextPutAll: aGitObject byteData.
	^stream contents! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:53'!
bytes
	"Answers the byte sequence that was used to calculate the sha1 name of the associated object."
	
	^signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/8/2010 20:41'!
computeSignatureFor: aGitObject
	signature := self byteSignatureFor: aGitObject.
	integerHash := self integerHashFrom: signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 13:32'!
from: aHexHash
	integerHash := Integer readFrom: aHexHash asString asUppercase base: 16! !

!GitObjectSignature methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:14'!
hash
	^ self integerHash! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:53'!
hexHash
	"Answers the sha1 name of the associated object."
	
	^self hexHashFrom: integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 14:38'!
hexHashFrom: aByteString
	"#hex will omit a leading 0. If that's the case insert leading 0."
	^aByteString asByteArray hex asLowercase forceTo: 40 paddingStartWith: $0! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:54'!
integerHash
"Answers the sha1 hash of the associated object in integer representation."

	^integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 4/29/2010 18:17'!
integerHashFrom: byteArray
	^SecureHashAlgorithm new hashMessage: byteArray! !

!GitObjectSignature methodsFor: 'printing' stamp: 'MaxLeske 5/8/2010 20:42'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self hexHash; nextPut: $).! !

!GitObjectSignature methodsFor: 'saving' stamp: 'MaxLeske 7/31/2010 17:51'!
saveTo: aStream
	aStream nextPutBytes: (self integerHash asByteArray forceTo: 20 paddingStartWith: 0)! !

GitObject subclass: #GitStorableObject
	instanceVariableNames: 'signature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStorableObject commentStamp: '<historical>' prior: 0!
I am an abstract class defining common messages to which all object (in git speak) respond that can actually be stored, i.e. commits, trees, tags, blobs.

Instance Variables:
	signature	<GitObjectSignature>
		Lazily loaded signature of the object. See class comment for GitObjectSignature.!

GitStorableObject subclass: #GitBlob
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitBlob commentStamp: '<historical>' prior: 0!
I am the git object responsible for storing content. I do not know or care what data I hold.

Instance Variables:
	bytes	<ByteArray>
		The contents I hold, stored in a ByteArray.!

!GitBlob class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 16:02'!
bytes: aByteArray
	^ self new initializeBytes: aByteArray asByteArray! !

!GitBlob class methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:19'!
typeCode
	^ 'blob'! !

!GitBlob methodsFor: 'saving' stamp: 'MaxLeske 7/23/2010 14:56'!
bytes
"Answers the contents of a blob object."

	^ bytes! !

!GitBlob methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 13:31'!
initializeBytes: aByteArray
	bytes := aByteArray! !

!GitBlob methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:15'!
loadFrom: aStream
	bytes := aStream contents.
	^self! !

!GitBlob methodsFor: 'saving' stamp: 'tonyg 2/23/2010 15:36'!
saveTo: aStream
	aStream nextPutAll: bytes.! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/26/2010 22:14'!
forType: aString  
 ^ self allSubclasses
       detect: [ :class | aString isNil not and: [class typeCode = aString] ]
       ifNone: [ self error: 'Invalid type code: ' , aString printString ]! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:45'!
loadFrom: aStream from: aGitRepository
	| code length limitedStream object |
	code := (aStream upTo: Character space asInteger) asString.
	length := (aStream upTo: 0) asString asInteger.
	limitedStream := ReadStream on: (aStream next: length).
	object := (self forType: code) in: aGitRepository.
	^object loadFrom: limitedStream! !

!GitStorableObject class methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 12:24'!
typeCode
	"Override in subclasses, returning non-nil where you want them to be constructible from a git object store."
	^ nil! !

!GitStorableObject methodsFor: 'comparing' stamp: 'MaxLeske 4/28/2010 21:25'!
= other
	^self signature = other signature! !

!GitStorableObject methodsFor: 'private' stamp: 'Max Leske 4/25/2010 13:05'!
byteData
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream contents! !

!GitStorableObject methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:17'!
hash
	^ self signature xor: self class name! !

!GitStorableObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:37'!
signature
	^signature ifNil: [ signature := GitObjectSignature for: self ]! !

!GitStorableObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:40'!
signature: aSignature
	signature := aSignature! !

!GitStorableObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 15:47'!
typeCode
	^ self class typeCode! !

GitStorableObject subclass: #GitTaglikeObject
	instanceVariableNames: 'message properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTaglikeObject commentStamp: '<historical>' prior: 0!
I am an abstract class defining common protocol for git objects that behave like tags.

Instance Variables:
	message	<String>
		Holds the message of the tag like object, if used.
	properties	<(Collection of: (Association of: String -> ByteArray))>
		Holds arbitrary key value pairs not specified by git that can be used by applications to store additional data.!

GitTaglikeObject subclass: #GitCommit
	instanceVariableNames: 'tree parents author committer isLoaded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitCommit commentStamp: '<historical>' prior: 0!
I represent the state of a git repository in time. It is mandatory to provide me with a message describing the changes.

Instance Variables:
	tree	<GitTree>
		Holds a tree representing the root of the repository.
	parents	<Collection of: GitCommit>
		Holds a collection of parent commits. A parent commit represents 
		the state the repository was in when the changes occurred that led to the current commit.
		In case of a merge it is possible for a commit to have more than one parent.
	author	<GitStamp>
		Denotes the author of the content committed.
	committer	<GitStamp>
		Denotes the committer of the commit. The committer might be a different
		person than the author.!

!GitCommit class methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 22:21'!
typeCode
	^ 'commit'! !

!GitCommit class methodsFor: 'instance creation' stamp: 'MaxLeske 7/24/2010 17:18'!
withDefaultStamps
	| s |
	s := GitStamp now.
	^ self new 
		author: s;
		committer: s;
		yourself! !

!GitCommit class methodsFor: 'instance creation' stamp: 'MaxLeske 5/25/2010 19:59'!
withTree: aGitTree message: aString author: aGitStampForAuthor committer: aGitStampForCommitter
	^self new
		tree: aGitTree;
		message: aString;
		author: aGitStampForAuthor;
		committer: aGitStampForCommitter;
		yourself
		! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:49'!
author
	^ author ifNil: [ 
		self isLoaded ifFalse: [ self loadLazily. author ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:52'!
author: aGitStamp
	author := aGitStamp! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'MaxLeske 7/26/2010 08:46'!
beLoaded: aBoolean
	isLoaded := aBoolean! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:49'!
committer
	^ committer ifNil: [ 
		self isLoaded ifFalse: [ self loadLazily. committer ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:51'!
committer: aGitStamp
	committer := aGitStamp! !

!GitCommit methodsFor: 'initialize-release' stamp: 'MaxLeske 7/26/2010 08:47'!
initialize
	super initialize.
	isLoaded := false! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'MaxLeske 7/26/2010 08:46'!
isLoaded
	^ isLoaded! !

!GitCommit methodsFor: 'loading' stamp: 'MaxLeske 7/26/2010 08:39'!
loadKey: k value: v
	k = 'tree' ifTrue: [tree := repository objectNamed: v. ^ self].
	k = 'parent' ifTrue: [
		parents ifNil: [parents := OrderedCollection new].
		parents add: ((GitCommit in: repository) signature: (GitObjectSignature from: v)).
		^ self].
	k = 'author' ifTrue: [author := GitStamp fromString: v. ^ self].
	k = 'committer' ifTrue: [committer := GitStamp fromString: v. ^ self].
	"Otherwise it's treated generically."
	^ super loadKey: k value: v! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'MaxLeske 7/29/2010 10:54'!
loadLazily
	| stored |
	self beLoaded: true.
	repository ifNil: [ 
		message ifNil: [ message := '' ].
		properties ifNil: [ properties := OrderedCollection new ].
		parents ifNil: [ parents := OrderedCollection new ].
		^ self ].
		
	stored := repository objectWithSignature: self signature.
	stored beLoaded: true.
	
	self 
		message: (stored message ifNil: [ '' ]);
		properties: (stored properties ifNil: [ OrderedCollection new ]);
		tree: stored tree;
		author: stored author;
		committer: stored committer;
		parents: (stored parents ifNil: [ OrderedCollection new ])! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:49'!
message
	^ message ifNil: [ 
		self isLoaded ifFalse: [ self loadLazily. message ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/29/2010 10:32'!
parents
	^ parents ifNil: [ 
		self isLoaded 
			ifTrue: [ parents := OrderedCollection new ]
			ifFalse: [ self loadLazily. parents ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 20:04'!
parents: aGitCommitCollection
	^ parents := aGitCommitCollection! !

!GitCommit methodsFor: 'printing' stamp: 'MaxLeske 7/9/2010 17:06'!
printOn: aStream
	super printOn: aStream.
	self message
		ifNotNil: [
			aStream
				nextPutAll: '(''';
				nextPutAll: self message;
				nextPutAll: ''')' ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:49'!
properties
	^ properties ifNil: [ 
		self isLoaded ifFalse: [ self loadLazily. properties ] ]! !

!GitCommit methodsFor: 'saving' stamp: 'MaxLeske 7/22/2010 19:40'!
saveFixedKeysTo: aStream
	"Looking at builtin-commit-tree.c, the order of properties is fixed:
	tree, parents, author, committer, everything else, message."
	self saveKey: GitTree typeCode value: tree signature hexHash to: aStream.
	self parents do: [:each | self saveKey: 'parent' value: each signature hexHash to: aStream].
	self saveKey: 'author' value: author printString to: aStream.
	self saveKey: 'committer' value: committer printString to: aStream.! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:49'!
tree
	^ tree ifNil: [ 
		self isLoaded ifFalse: [ self loadLazily. tree ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:49'!
tree: aGitTree
	tree := aGitTree! !

!GitCommit methodsFor: 'validating' stamp: 'MaxLeske 5/26/2010 21:47'!
validate
	tree ifNil: [self error: 'Cannot have a commit without a tree.'].
	tree species = GitTree ifFalse: [self error: 'Commit needs to hold an instance of a GitTree.'].
	author ifNil: [self error: 'Author identity is required.'].
	committer ifNil: [self error: 'Committer identity is required.'].
	super validate.! !

GitTaglikeObject subclass: #GitTag
	instanceVariableNames: 'tagger object name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTag commentStamp: '<historical>' prior: 0!
I have the responsibility of associating a human readable tag (a string) with a commit (on the filesystem level I do this by simply referencing the sha1 name of a commit), e.g. 'version 0.5' -> '2341f8c0615bbcc465ac4686025e880786430697'.

It is important to know that there are light tags and real tag objects in git. Light tags are represented through a file with the tag name that holds a reference to a commit. Alternatively that file can hold a reference to a tag object which in turn references a commit. Full tags have the advantage of knowing the tagger (and the date of tagging) and can hold a message (e.g. a PGP signature).
I incorporate both possibilities.

Instance Variables:
	tagger	<GitStamp>
		Who created the tag.
	object	<GitCommit>
		The commit referenced by the tag name.
	name	<String>
		The human readable tag (or tag name) that describes the referenced commit.!

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 7/24/2010 17:19'!
makeLightTagNamed: aString for: aGitCommit in: aGitRepository
	| tag |
	self assert: aGitCommit species = GitCommit.
	tag := self new
		name: aString;
		object: aGitCommit;
		repository: aGitRepository;
		yourself.
	tag store.
	^tag
		! !

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 6/30/2010 15:32'!
makeTagNamed: tagName for: aGitCommit message: message in: aGitRepository
	^ self
		makeTagNamed: tagName
		for: aGitCommit
		message: message
		tagger: GitStamp now
		in: aGitRepository! !

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 7/24/2010 17:19'!
makeTagNamed: tagName for: aGitCommit message: message tagger: aGitStamp in: aGitRepository
	| tag |
	tag := self new
		name: tagName;
		object: aGitCommit;
		message: message;
		tagger: aGitStamp;
		repository: aGitRepository
		yourself.
	tag store.
	^tag! !

!GitTag class methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:16'!
typeCode
	^ 'tag'! !

!GitTag methodsFor: 'testing' stamp: 'MaxLeske 6/30/2010 14:56'!
isLightTag
	^self message isNil and: [self tagger isNil]! !

!GitTag methodsFor: 'loading' stamp: 'MaxLeske 6/30/2010 15:30'!
loadKey: k value: v
	k = 'object' ifTrue: [object := repository objectNamed: v. ^ self].
	k = 'type' ifTrue: [self assert: object typeCode = v. ^ self].
	k = 'tag' ifTrue: [name := v. ^ self].
	k = 'tagger' ifTrue: [tagger := GitStamp fromString: v. ^ self].
	"Otherwise it's treated generically."
	^ super loadKey: k value: v! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 15:11'!
name
	^ name! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 13:59'!
name: aString
	name := aString! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 15:12'!
object
	^object! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:01'!
object: aGitCommit
	self assert: aGitCommit species = GitCommit.
	object := aGitCommit! !

!GitTag methodsFor: 'printing' stamp: 'MaxLeske 7/18/2010 14:41'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: '("';
		nextPutAll: self name;
		nextPutAll: '")'! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 6/30/2010 15:13'!
saveFixedKeysTo: aStream
	"From builtin-mktag.c:
	A signature file has a very simple fixed format: four lines
	of ""object <sha1>"" + ""type <typename>"" + ""tag <tagname>"" +
	""tagger <committer>"", followed by a blank line, a free-form tag
	message and a signature block that git itself doesn't care about,
	but that can be verified with gpg or similar."
	self saveKey: 'object' value: self object signature hexHash to: aStream.
	self saveKey: 'type' value: self object typeCode to: aStream.
	self saveKey: 'tag' value: self name to: aStream.
	self saveKey: 'tagger' value: self tagger printString to: aStream.! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 6/30/2010 14:53'!
store
	self isLightTag
		ifTrue: [self storeLightTag]
		ifFalse: [self storeFullTag]
	! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 7/24/2010 15:25'!
storeFullTag
	repository 
		updateRef: repository tagsDir / self name 
		to: (repository storeObject: self) signature! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 7/24/2010 15:25'!
storeLightTag
	repository 
		updateRef: repository tagsDir / self name 
		to: self object signature! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagger
	^ tagger! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:00'!
tagger: aGitStamp
	self assert: aGitStamp species = GitStamp.
	tagger := aGitStamp! !

!GitTag methodsFor: 'validating' stamp: 'MaxLeske 6/30/2010 14:00'!
validate
	self object ifNil: [self error: 'Cannot have a tag without a tagged object.'].
	self name ifNil: [self error: 'Tag name is required.'].
	self tagger ifNil: [self error: 'Tagger identity is required.'].
	super validate.! !

!GitTaglikeObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 21:53'!
loadFrom: aStream
	| key value |
	[aStream peek = Character lf asInteger] whileFalse: [
		key := (aStream upTo: Character space asInteger) asString.
		value := (aStream upTo: Character lf asInteger) asString.
		self loadKey: key value: value].
	aStream next.
	message := aStream upToEnd asString.
	^self
! !

!GitTaglikeObject methodsFor: 'loading' stamp: 'MaxLeske 6/26/2010 16:15'!
loadKey: k value: v
	self properties add: k -> v.! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:22'!
message
	^ message! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:22'!
message: aString
	message := aString! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:22'!
properties
	^ properties ifNil: [properties := OrderedCollection new]! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:23'!
properties: anObject
	^ properties := anObject! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'tonyg 2/25/2010 16:27'!
saveFixedKeysTo: aStream
	self subclassResponsibility! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'MaxLeske 4/23/2010 13:50'!
saveKey: key value: value to: aStream
	aStream 
		nextPutAll: key; space; 
		nextPutAll: value; lf.! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'MaxLeske 6/26/2010 16:38'!
saveTo: aStream
	self validate.
	self saveFixedKeysTo: aStream.
	properties ifNotNil: [properties do: [:assoc |
			self saveKey: assoc key value: assoc value to: aStream]].
	aStream lf.
	aStream nextPutAll: message.! !

!GitTaglikeObject methodsFor: 'validating' stamp: 'MaxLeske 7/23/2010 14:58'!
validate
	"Override in subclasses to provide pre-save validation checks.
	Don't forget to call super validate, though!!"
	
	message ifNil: [self error: 'Message is required.'].! !

GitStorableObject subclass: #GitTree
	instanceVariableNames: 'entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTree commentStamp: '<historical>' prior: 0!
I represent a node in a git repository that can contain child nodes (mostly I simply represent a directory).

Instance Variables:
	entries	<aCollection of: GitTreeEntry>
		Holds references to all the child nodes together with some 
		additional information (see the class comment for GitTreeEntry).!

!GitTree class methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	^self new entries: aCollection! !

!GitTree class methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:17'!
typeCode
	^ 'tree'! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:38'!
at: aName
	^ repository objectWithSignature: (self entryAt: aName) objectSignature! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:28'!
entries
	^ entries ifNil: [Dictionary new]! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	aCollection do: [:each | self putEntry: each].! !

!GitTree methodsFor: 'private' stamp: 'tonyg 2/23/2010 19:10'!
entryAt: aName
	^ entries at: aName! !

!GitTree methodsFor: 'private' stamp: 'MaxLeske 10/11/2010 23:14'!
entryAt: aName ifAbsent: aBlock
	^ entries 
		at: aName
		ifAbsent: aBlock! !

!GitTree methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 20:18'!
initialize
	super initialize.
	entries := Dictionary new.! !

!GitTree methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:35'!
loadFrom: aStream
	| entry |
	[aStream atEnd] 
		whileFalse: [entry := (GitTreeEntry in: repository) loadFrom: aStream.
			self putEntry: entry].
	^self! !

!GitTree methodsFor: 'private' stamp: 'MaxLeske 7/10/2010 10:17'!
putEntry: anEntry
	entries at: anEntry entryName put: anEntry! !

!GitTree methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:14'!
removeEntryAt: aFileName
	entries removeKey: aFileName! !

!GitTree methodsFor: 'saving' stamp: 'MaxLeske 7/9/2010 17:37'!
saveTo: aStream
	entries values sorted do: [:e | e saveTo: aStream].! !

!GitTree methodsFor: 'testing' stamp: 'MaxLeske 6/8/2010 22:36'!
size
	| total |
	total := 0.
	self entries do: [:each | total := total + (each size)].
	^total! !

GitObject subclass: #GitTreeEntry
	instanceVariableNames: 'mode objectSignature entryName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTreeEntry commentStamp: '<historical>' prior: 0!
I represent an entry for a child node of a GitTree. I provide additionally information about the object (either another tree or a blob) I reference.

Instance Variables:
	mode	<Integer>
		Octal representation of the type of entry ('chmod' bit settings).
		The mode for newly created objects is either that of 'self fileMode' or 'self dirMode'.
		Loaded entries may have a varying bit sequence because git only pays attention
		to the executable bit.
	objectSignature	<GitObjectSignature>
		The signature of the object referenced.
	entryName	<String>
		'Physical' name of the entry. On a filesystem this would be the name of the
		file or directory.!

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 6/23/2010 19:10'!
dirMode
	^8r040000! !

!GitTreeEntry class methodsFor: 'instance creation' stamp: 'MaxLeske 7/10/2010 10:39'!
entryName: aString mode: anInteger objectSignature: aGitObjectSignature
	^ self new initializeFileName: aString mode: anInteger signature: aGitObjectSignature! !

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 6/23/2010 19:13'!
fileMode
	^8r100644! !

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 7/21/2010 18:59'!
modeFor: aReference
	^ aReference isDirectory 
		ifTrue: [ self dirMode ]
		ifFalse: [ self fileMode ]! !

!GitTreeEntry class methodsFor: 'printing' stamp: 'MaxLeske 7/10/2010 11:10'!
printMode: octalMode
	^octalMode printStringBase: 8! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:04'!
<= other
	"For sorting reasons, directories are compared
	with a virtual slash appended."
	| myTail otherTail |
	myTail := otherTail := ''.
	self mode = (self class dirMode)
		ifTrue: [ myTail := '/' ].
	other mode = (self class dirMode)
		ifTrue: [otherTail := '/' ].
	
	^ self entryName, myTail <= (other entryName, otherTail)! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 8/5/2010 17:15'!
= other
	^ other species = self species and: [
		other objectSignature = self objectSignature ]! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 10/12/2010 17:49'!
beClean
	dirty := false! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 7/10/2010 10:16'!
entryName
	^ entryName! !

!GitTreeEntry methodsFor: 'testing' stamp: 'MaxLeske 10/12/2010 19:37'!
hasDirectoryMode
	^ self hasFileMode not! !

!GitTreeEntry methodsFor: 'testing' stamp: 'MaxLeske 10/12/2010 19:36'!
hasFileMode
	^ self mode = (self species fileMode)! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 10/12/2010 22:16'!
hash
	^ self species hash bitXor: (self mode bitXor: entryName hash)! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'MaxLeske 10/12/2010 17:49'!
initialize
	super initialize.
	dirty := false! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'MaxLeske 7/10/2010 10:16'!
initializeFileName: aString mode: anInteger signature: aGitObjectSignature
	entryName := aString.
	mode := anInteger.
	objectSignature := aGitObjectSignature! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'MaxLeske 7/10/2010 10:16'!
loadFrom: aStream
	mode := Integer readFrom: (ReadStream on: (aStream upTo: Character space asInteger) asString) base: 8.
	entryName := (aStream upTo: 0) asString.
	objectSignature := GitObjectSignature loadFrom: aStream.
	^self! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 08:08'!
mode
	^ mode! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 13:03'!
objectSignature
	^ objectSignature! !

!GitTreeEntry methodsFor: 'printing' stamp: 'MaxLeske 6/23/2010 19:17'!
printMode
	^self class printMode: self mode! !

!GitTreeEntry methodsFor: 'printing' stamp: 'MaxLeske 7/10/2010 10:47'!
printOn: aStream
	super printOn: aStream.
	self entryName
		ifNotNil: [
			aStream
				nextPutAll: '(''';
				nextPutAll: self entryName;
				nextPutAll: ''')' ]! !

!GitTreeEntry methodsFor: 'saving' stamp: 'MaxLeske 7/10/2010 10:16'!
saveTo: aStream.
	aStream 
		nextPutAll: self printMode;
		space;
		nextPutAll: entryName;
		nullByte.
	objectSignature saveTo: aStream.! !

Object subclass: #GitObjectLoader
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObjectLoader commentStamp: '<historical>' prior: 0!
I am a helper class that manages loading of objects. I was created to relieve repository objects and GitObjects from having to know about loading protocol.

Instance Variables:
	repository	<GitRepository>
		A reference to the repository an object should be loaded from.!

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:03'!
for: aGitRepository
	^self basicNew initializeOn: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:04'!
new
	self error: 'please use the constructor methods.'! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 12:11'!
objectFrom: aStream from: aGitRepository
	^self basicNew objectFrom: aStream from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 11:28'!
objectNamed: aGitObjectSignature from: aGitRepository
	^self basicNew objectNamed: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 13:16'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	^self basicNew objectWithSignature: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:04'!
initializeOn: aGitRepository
	repository := aGitRepository! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 17:58'!
objectFrom: aStream
	repository ifNil: [self error: 'Instance variable #repository needs to be initialized.'].
	^GitStorableObject loadFrom: aStream from: repository! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 16:57'!
objectFrom: aStream from: aGitRepository
	repository := aGitRepository.
	^self objectFrom: aStream! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:43'!
objectNamed: aHexHash
	repository ifNil: [self error: 'instance variable #repository needs to be initialized.'].
	^self objectWithSignature: (GitObjectSignature from: aHexHash)! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:47'!
objectNamed: aHexHash from: aGitRepository
	repository := aGitRepository.
	^self objectNamed: aHexHash! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:53'!
objectWithSignature: aGitObjectSignature
	repository ifNil: [self error: 'Instance variable #repository needs to be initialized.'].
	
	^ [ self objectFrom: (repository looseObjectStreamAt: aGitObjectSignature) ] 
		on: FSFileDoesNotExist
		do: [ :ex | ex return: (repository packedObjectWithSignature: aGitObjectSignature) object ]! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:44'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	repository := aGitRepository.
	^self objectWithSignature: aGitObjectSignature! !

Object subclass: #GitPack
	instanceVariableNames: 'idx basePath fanoutTable objectCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 15:42'!
basePath
	^ basePath! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:27'!
basePath: aReference
	basePath := aReference.
	(basePath , 'idx') readStreamDo: [ :stream | idx := ReadStream on: stream contents ].
	((idx next: 8) = #(255 116 79 99 0 0 0 2) asByteArray)
		ifFalse: [self error: 'Bad pack index header'].
	fanoutTable := idx next: 1024. "256 4-byte big-endian entries."
	objectCount := self fanoutTableAt: 255.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:43'!
crcAt: index
	"1024 fanout, 8 header, minus 4 for smalltalk's 1-based indexing"
	idx position: 1028 + (objectCount * 20) + (index * 4).
	^ (idx next: 4) asInteger.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:31'!
fanoutTableAt: fanoutIndex
	^ fanoutTable unsignedLongAt: (fanoutIndex * 4) + 1 bigEndian: true.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/24/2010 00:32'!
objectAt: sha1String
	| p |
	p := self packedObjectAt: sha1String.
	p ifNotNil: [^ p object].
	^ nil! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:47'!
offsetAt: index
	| o |
	"1024 fanout, 8 header. We're still using zero-based indexing here."
	idx position: 1032 + (objectCount * 24) + (index * 4).
	o := Integer readFrom: (idx next: 4) hex base: 16.
	(o >= 16r80000000)
		ifFalse: [^ o]
		ifTrue: [
			"the table offset is zero-based here because it comes from git, not smalltalk"
			idx position: 1032 + (objectCount * 28) + ((o bitAnd: 16r7FFFFFFF) * 8).
			^ (idx next: 8) asInteger]! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:56'!
packedObjectAt: sha1String
	| i o |
	i := self tableIndexForSha: sha1String.
	i ifNil: [^ nil].
	o := self offsetAt: i.
	^ self packedObjectAtOffset: o! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:51'!
packedObjectAtOffset: offset 
	| p c |
	(basePath , 'pack') readStreamDo: [ :stream | p := ReadStream on: stream contents ].
	p position: offset.
	c := GitPackedObject new pack: self.
	c loadFrom: p.
	^ c! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:47'!
tableIndexForSha: s
	| fanoutIndex sNum index low high test |
	sNum := Integer readFrom: (ReadStream on: s asUppercase) base: 16.
	fanoutIndex := sNum bitShift: -152. "160 bit hash. We want the upper 8 bits."

	"We are searching using a zero-based index, since position: on the file is zero-based."
	low := (fanoutIndex > 0) ifTrue: [self fanoutTableAt: fanoutIndex - 1] ifFalse: [0].
	high := (self fanoutTableAt: fanoutIndex) - 1. "binary-search wants an inclusive range"

	[index := high + low // 2.
	low > high] whileFalse:[
		idx position: index * 20 + 1032. "256 * 4 for the fanoutTable, plus 8 for the header"
		test := Integer readFrom: (idx next: 20) hex base: 16.
		test = sNum 
			ifTrue:[^index]
			ifFalse:[sNum > test
				ifTrue: [low := index + 1]
				ifFalse: [high := index - 1]]].
	^ nil! !

Object subclass: #GitPackedObject
	instanceVariableNames: 'pack file decodeClass length headerPos unpackedBytes repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
applyDelta: delta to: base
	decodeClass := base decodeClass.
	unpackedBytes := delta runAgainst: base unpackedBytes.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
decodeClass
	^ decodeClass! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:04'!
loadFrom: aStream
	| b type shift |
	file := aStream.
	headerPos := file position.
	b := file next.
	type := (b bitShift: -4) bitAnd: 7.
	length := b bitAnd: 15.
	shift := 4.
	[(b bitAnd: 128) isZero] whileFalse: [
		b := file next.
		length := length bitOr: ((b bitAnd: 127) bitShift: shift).
		shift := shift + 7].
	self unpackContentType: type.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:55'!
object
	^ (decodeClass in: repository) loadFrom: (ReadStream on: unpackedBytes)! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack
	^ pack! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack: anObject
	pack := anObject! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:07'!
readDelta
	^ GitDeltaInterpreter new instructions: (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:54'!
repository: aRepository
	repository := aRepository! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:04'!
unpackContentType: type
	type = 1 "OBJ_COMMIT" ifTrue: [decodeClass := GitCommit. ^ self unpackSimpleContent].
	type = 2 "OBJ_TREE" ifTrue: [decodeClass := GitTree. ^ self unpackSimpleContent].
	type = 3 "OBJ_BLOB" ifTrue: [decodeClass := GitBlob. ^ self unpackSimpleContent].
	type = 4 "OBJ_TAG" ifTrue: [self error: 'Packed tags aren''t supported yet.'].
	type = 6 "OBJ_OFS_DELTA" ifTrue: [^ self unpackOfsDeltaContent].
	type = 7 "OBJ_REF_DELTA" ifTrue: [^ self unpackRefDeltaContent].
	self error: 'Invalid pack object type ', type printString.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:07'!
unpackOfsDeltaContent
	| offset b base delta |
	b := file next.
	offset := b bitAnd: 127.
	[(b bitAnd: 128) isZero] whileFalse: [
		b := file next.
		"This is bizarre, but it's what pack-format.txt and the Git source says is happening."
		offset := ((offset + 1) bitShift: 7) bitOr: (b bitAnd: 127)].

	delta := self readDelta.
	base := pack packedObjectAtOffset: headerPos - offset.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 22:12'!
unpackRefDeltaContent
	| s delta base |
	s := (GitObjectSignature loadFrom: file) resolve.
	delta := self readDelta.
	base := pack packedObjectAt: s.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 20:05'!
unpackSimpleContent 
	unpackedBytes := (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
unpackedBytes
	^ unpackedBytes! !

Object subclass: #GitRepository
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitRepository commentStamp: '<historical>' prior: 0!
I am responsible for managing low level operations on a repository. I keep a reference to the location of the physical repository.

Instance Variables
	reference:		<FSReference>
		Holds a reference to the physical location of the repository.
		
!

!GitRepository class methodsFor: 'instance creation' stamp: 'MaxLeske 4/13/2010 15:11'!
new
	self error: 'This class should be istantiated with a memory type.'! !

!GitRepository class methodsFor: 'initialize-release' stamp: 'MaxLeske 7/24/2010 17:05'!
on: aReference
	^self basicNew initializeOn: aReference
	! !

!GitRepository methodsFor: 'comparing' stamp: 'MaxLeske 6/30/2010 08:13'!
= other
	^self baseDir = (other baseDir)! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:04'!
baseDir
"Answers the reference to the actual repository location. If e.g. the repository was initialized with a reference to '/foo' but the repository were actually at '/foo/.git' then this method would answer a reference to '/foo/.git'".

	(reference basename endsWith: '.git')
		ifTrue: [^reference]
		ifFalse: [^reference resolve: '.git']! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 7/23/2010 16:57'!
basicUpdateRef: refPath to: aString
	| target pathReference |
	target := [(self baseDir resolve: refPath) readStreamDo: [ :stream | stream contents ]]
		on: FSFileDoesNotExist do: [:ex | ex return: nil].

	(target isNil not and: [target beginsWith: 'ref:']) ifTrue: [
		^ self updateRef: (target allButFirst: 4) withBlanksTrimmed to: aString ].

	pathReference := self baseDir resolve: refPath.
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ :stream | 
		stream 
			nextPutAll: aString; 
			nextPut: Character lf ]! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/28/2010 12:16'!
branches
"Answers a dictionary of all GitCommit objects referenced in '/refs/heads'."

	| dictionary |
	dictionary := Dictionary new.
	self headsDir children do: [ :each |
		dictionary at: each basename put: 
			(each readStreamDo: [ :stream |
				self objectNamed: stream contents asString ]) ].
	
	^ dictionary! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 6/30/2010 08:10'!
createReferencedDirectory
	| refWalker dirList |
	refWalker := self baseDir.
	dirList := OrderedCollection new.
	
	[refWalker ~= (self baseDir filesystem root)]
		whileTrue: [dirList addFirst: refWalker. refWalker := refWalker parent].
	
	dirList do: [:each | each exists ifFalse: [each createDirectory]]
	! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/24/2010 15:13'!
delete
	"Deletes the repository irrevocably."
	
	GitRepositoryDeleteRequest signal
		ifTrue: [ self baseDir delete ]! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 6/30/2010 08:15'!
exists
	^self baseDir exists and: [self isInitialized]! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 7/11/2010 12:32'!
gitDirectories
	^#('objects' 
		'objects/pack' 
		'objects/info' 
		'refs'
		'refs/heads' 
		'refs/tags' 
		'info')! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 6/30/2010 08:10'!
gitInitialize
	self exists
		ifTrue: [ ^ self ].
	self createReferencedDirectory.
	self gitDirectories do: [ :dir | (self baseDir resolve: dir) createDirectory ].
	self baseDir / 'HEAD'
		writeStreamDo: [ :stream | 
			stream
				nextPutAll: 'ref: refs/heads/master';
				nextPut: Character lf ]! !

!GitRepository methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:16'!
hash
	^ reference basename hash xor: self class name! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:14'!
head
"Answers the GitCommit object referenced in / via 'HEAD'."

	| signature |
	signature := self resolveRef: 'HEAD'.
	signature
		ifNotNil: [ ^ self objectWithSignature: signature ]
		ifNil: [ ^ nil ]! !

!GitRepository methodsFor: 'locations' stamp: 'MaxLeske 7/23/2010 18:02'!
headFile
	^ self baseDir resolve: 'HEAD'! !

!GitRepository methodsFor: 'locations' stamp: 'MaxLeske 7/23/2010 17:47'!
headsDir
	^ self baseDir resolve: 'refs/heads'! !

!GitRepository methodsFor: 'initialize-release' stamp: 'MaxLeske 7/23/2010 09:59'!
initializeOn: anFSReference
	reference := anFSReference.
	self gitInitialize! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 7/23/2010 18:13'!
isInitialized
	| isInitialized foundDirectories |
	foundDirectories := self gitDirectories collect: [:each | (self baseDir resolve: each) exists].
	isInitialized := foundDirectories size = (self gitDirectories size).
	isInitialized := self headFile exists.
	^isInitialized! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 5/26/2010 22:25'!
isObjectDirectory: aReference
	^aReference isDirectory and: [aReference basename size = 2]! !

!GitRepository methodsFor: 'enumerating' stamp: 'MaxLeske 6/30/2010 08:15'!
looseObjectNamesDo: aBlock 
	| root hash |
	root := self baseDir resolve: 'objects'.
	root childDirectories do: 
		[ :dirReference | (self isObjectDirectory: dirReference)
			ifTrue: [ dirReference childFiles do: 
				[ :fileReference | 
				hash := dirReference basename , fileReference basename.
				aBlock value: (GitObjectSignature from: hash) ] ] ]! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 6/27/2010 21:33'!
looseObjectPathFor: aGitObjectSignature
	| hexHash |
	hexHash := aGitObjectSignature hexHash.
	^ 'objects/' , (hexHash first: 2) , '/' , (hexHash allButFirst: 2)! !

!GitRepository methodsFor: 'private' stamp: 'lr 7/18/2010 18:04'!
looseObjectStreamAt: aGitObjectSignature
	^ (self baseDir resolve: (self looseObjectPathFor: aGitObjectSignature)) 
		readStreamDo: [ :stream | ReadStream on: (ZLibReadStream on: stream contents) contents ]! !

!GitRepository methodsFor: 'loading' stamp: 'MaxLeske 6/26/2010 16:48'!
objectNamed: aHexHash
	^GitObjectLoader objectNamed: aHexHash from: self! !

!GitRepository methodsFor: 'loading' stamp: 'MaxLeske 7/8/2010 16:52'!
objectWithSignature: aGitObjectSignature
	^GitObjectLoader objectWithSignature: aGitObjectSignature from: self! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/26/2010 12:55'!
packedObjectWithSignature: aGitObjectSignature
	| object |
	self packsDo: [ :each |
		object := each packedObjectAt: aGitObjectSignature hexHash.
		object ifNotNil: [^ object repository: self] ].
	^ nil! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/26/2010 11:55'!
packedRefs
	| packedRefs dictionary hexHash refName stream |
	packedRefs := self baseDir resolve: 'packed-refs'.
	packedRefs exists ifFalse: [^ nil].
	
	dictionary := Dictionary new.
	[ stream := ReadStream on: packedRefs readStream contents asString ]
		ensure: [ stream close ].
	[ stream atEnd ] whileFalse: [
		stream peek = $#
			ifTrue: [ stream upTo: Character lf ]
			ifFalse: [
				hexHash := stream upTo: Character space.
				refName := stream upTo: Character lf.
				dictionary at: refName put: (GitObjectSignature from: hexHash) ] ].
	^ dictionary! !

!GitRepository methodsFor: 'enumerating' stamp: 'MaxLeske 7/26/2010 12:21'!
packsDo: aBlock
	| dir shortName |
	dir := self baseDir resolve: 'objects/pack'.
	dir childFiles 
		do: [ :each | (each basename endsWith: '.idx')
			ifTrue: [ shortName := each basename allButLast: 4.
					aBlock value: (GitPack new basePath: (dir resolve: shortName))]]! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 17:43'!
resolveLooseRef: refPath
	| target |
	target := [ 
		(self baseDir resolve: refPath) 
			readStreamDo: [ :stream | stream contents asString ] ]
		on: FSFileDoesNotExist 
		do: [:ex | ^ nil].
	(target beginsWith: 'ref:') ifTrue: [^ self resolveRef: (target allButFirst: 4) withBlanksTrimmed].
	^ GitObjectSignature from: target substrings first "Ignore other junk after the hash."! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/26/2010 11:59'!
resolveRef: name
"From the git-rev-parse manpage:
            1. if $GIT_DIR/<name> exists, that is what you mean (this is usually useful only for HEAD, FETCH_HEAD, ORIG_HEAD and MERGE_HEAD);
            2. otherwise, $GIT_DIR/refs/<name> if exists;
            3. otherwise, $GIT_DIR/refs/tags/<name> if exists;
            4. otherwise, $GIT_DIR/refs/heads/<name> if exists;
            5. otherwise, $GIT_DIR/refs/remotes/<name> if exists;
            6. otherwise, $GIT_DIR/refs/remotes/<name>/HEAD if exists."
	| fullName havePackedRefs packedRefs result |	
	havePackedRefs := false.
	packedRefs := nil.
	#(	''
		'refs/'
		'refs/tags/'
		'refs/heads/'
		'refs/remotes/'
		"We don't do the last case (number 6) from the manual yet."
	) do: [:prefix |
		fullName := self baseDir path asString , '/' , prefix , name.
		result := self resolveLooseRef: fullName.
		result ifNotNil: [^ result].
		havePackedRefs ifFalse: [packedRefs := self packedRefs. havePackedRefs := true].
		packedRefs ifNotNil: [
			result := self packedRefs at: prefix , name ifAbsent: [ nil ].
			result ifNotNil: [ ^ result ] ] ].
	^ nil! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 6/30/2010 08:16'!
storeLooseObject: aGitStorableObject
	| zipStream compressed pathReference |
	zipStream := ZLibWriteStream on: (ByteArray new).
	zipStream nextPutAll: aGitStorableObject signature bytes; close.
	compressed := zipStream encodedStream contents.
	
	pathReference := self baseDir resolve: (self looseObjectPathFor: aGitStorableObject signature).
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ :stream | stream nextPutAll: compressed ].
	^aGitStorableObject! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:15'!
storeObject: anObject
"Stores a GitStorableObject in the repository and answers it."

	^self storeLooseObject: anObject! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/24/2010 17:27'!
tags
"Answers a collection of all GitTag objects referenced in '/refs/tags'. Light tags are also represented through GitTag objects (despite the direct reference to a commit)."

	| referencedObjects tags |
	referencedObjects := Dictionary new.
	self tagsDir children do: [ :ref |
		ref readStreamDo: [ :stream |
			referencedObjects 
				at: ref basename
				put: (self objectNamed: stream contents asString) ] ].

	tags := OrderedCollection new.
	referencedObjects associationsDo: [ :assoc |
		assoc value species = GitCommit
			ifTrue: [ 
				tags add: (GitTag new
					name: assoc key;
					object: assoc value;
					tagger: assoc value committer;
					yourself) ]
			ifFalse: [ tags add: assoc value ] ].
	^ tags! !

!GitRepository methodsFor: 'locations' stamp: 'MaxLeske 7/23/2010 17:47'!
tagsDir
	^ self baseDir resolve: 'refs/tags'! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 18:21'!
updateHeadToBranch: branchName
"Updates the reference in 'HEAD' to branchName"

	self basicUpdateRef: 'HEAD' to: 'ref: /refs/heads/' , branchName! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 16:58'!
updateRef: refPath to: aGitObjectSignature
"Updates the file denoted by refPath to reference the object with the sha1 name in aGitObjectSignature.
If the file denoted by refPath does not yet exist it will be created."

	self basicUpdateRef: refPath to: aGitObjectSignature hexHash! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:10'!
workingDir
"Answers the reference the repository object was initialized on (as opposed to were the repository is located; see #baseDir)."

	^reference! !

Object subclass: #GitStamp
	instanceVariableNames: 'name email timeStamp'
	classVariableNames: 'DefaultEmail DefaultName'
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStamp commentStamp: '<historical>' prior: 0!
I am responsible for providing a name, an e-mail address and a timestamp in the format specified by git.

Instance Variables:
	name	<String>
		First and last name of the person referenced.
	email	<String>
		E-mail address of the person referenced.
	timeStamp	<TimeStamp>
		A timestamp denoting the time the operation this stamp is associated
		with was performed (e.g. a commit).!

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 7/13/2010 15:35'!
defaultEmail
	DefaultEmail isEmptyOrNil 
		ifTrue: [ ^ GitStampEmailRequest signal ]
		ifFalse: [^DefaultEmail].
	^self defaultEmail! !

!GitStamp class methodsFor: 'accessing' stamp: 'lr 4/13/2010 16:20'!
defaultEmail: aString
	DefaultEmail := aString! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 7/13/2010 15:35'!
defaultName
	DefaultName isEmptyOrNil 
		ifTrue: [ ^ GitStampNameRequest signal ]
		ifFalse: [^DefaultName].
	^self defaultName! !

!GitStamp class methodsFor: 'accessing' stamp: 'lr 4/13/2010 16:21'!
defaultName: aString
	DefaultName := aString! !

!GitStamp class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 21:29'!
fromString: aString
	^ self readFrom: (ReadStream on: aString)! !

!GitStamp class methodsFor: 'instance creation' stamp: 'lr 4/13/2010 16:50'!
now
	^ self new 
		name: self defaultName; 
		email: self defaultEmail;
		yourself! !

!GitStamp class methodsFor: 'instance creation' stamp: 'MaxLeske 7/19/2010 19:11'!
readFrom: aStream
	| n e s sign z |
	n := (aStream upTo: $<) withBlanksTrimmed.
	e := (aStream upTo: $>) withBlanksTrimmed.
	aStream skipSeparators.
	s := Integer readFrom: aStream.
	aStream skipSeparators.
	sign := aStream next.
	('-+' includes: sign) ifFalse: [self error: 'Invalid timezone sign'].
	z := Integer readFrom: aStream.
	z := (z // 100) hours + (z \\ 100) minutes.
	sign = $- ifTrue: [z := z negated].
	
	^ self new
		name: n;
		email: e; 
		timeStamp: (((DateAndTime fromUnixTime: s) + z) offset: z);
		yourself! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 6/29/2010 15:58'!
requestEmail
	^FillInTheBlankMorph 
		request: 'Please provide a valid e-mail address so that you can be contacted if necessary.'
		initialAnswer: 'user@domain.ext'! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 6/29/2010 15:58'!
requestName
	^FillInTheBlankMorph 
		request: 'Please provide your full name to identify your actions.'
		initialAnswer: 'Firstname Lastname'! !

!GitStamp methodsFor: 'private' stamp: 'tonyg 2/23/2010 21:24'!
cleanUp: aString
	^ aString copyWithoutAll: '<>', String lf! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
email
	^ email! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:23'!
email: aString
	email := self cleanUp: aString! !

!GitStamp methodsFor: 'initialize-release' stamp: 'lr 4/13/2010 16:46'!
initialize
	super initialize.
	timeStamp := TimeStamp now! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:22'!
name
	^ name! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:23'!
name: aString
	name := self cleanUp: aString! !

!GitStamp methodsFor: 'printing' stamp: 'lr 7/19/2010 13:48'!
printOn: aStream
	| offset |
	aStream
		nextPutAll: name;
		nextPutAll: ' <'; nextPutAll: email; nextPutAll: '> ';
		nextPutAll: timeStamp asUnixTime printString;
		nextPut: Character space.
	offset := timeStamp offset.
	"a negative Duration will return a negative SmallInteger for e.g. #hours.
	To print our custom string we need to prepend our own $- and work with a
	positive Duration."
	offset negative
		ifTrue: [aStream nextPut: $-. offset := offset negated]
		ifFalse: [aStream nextPut: $+].
	aStream
		nextPutAll: (offset hours printString forceTo: 2 paddingStartWith: $0);
		nextPutAll: (offset minutes printString forceTo: 2 paddingStartWith: $0).! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
timeStamp
	^ timeStamp! !

!GitStamp methodsFor: 'accessing' stamp: 'MaxLeske 5/25/2010 20:30'!
timeStamp: aTimeStamp
	timeStamp := aTimeStamp! !

Object subclass: #GitStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStream commentStamp: '<historical>' prior: 0!
I am a wrapper for a WriteStream on a ByteArray. I provide convenience conversions for certain content that is written to the stream to simplify code (e.g. converting a String argument to a ByteArray before writing it on the stream).

Instance Variables:
	stream	<WriteStream>
		The internally used WriteStream!

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
contents
	^stream contents! !

!GitStream methodsFor: 'initialize-release' stamp: 'MaxLeske 4/23/2010 13:45'!
initialize
	stream := WriteStream on: ByteArray new! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 15:25'!
lf
"Appends the linefeed character to the stream."

	self nextPut: Character lf! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:43'!
nextPut: aCharacter
	stream nextPut: aCharacter asInteger! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
nextPutAll: aString
	stream nextPutAll: aString asByteArray ! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 10:55'!
nextPutBytes: aByteArray
	stream nextPutAll: aByteArray! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:51'!
nullByte
	self nextPut: 0! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 6/8/2010 22:33'!
size
	^stream size! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
space
	self nextPut: Character space! !
